# Copyright 2016 Huawei Technologies India Pvt. Ltd.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from neutronclient._i18n import _
from neutronclient.common import utils
from neutronclient.common import validators
from neutronclient.neutron import v2_0 as neutronv20
from neutronclient.neutron.v2_0.bgp import peer as bgp_peer

# Allowed BGP Autonomous number range
MIN_AS_NUM = 1
MAX_AS_NUM = 4294967295


def get_network_id(client, id_or_name):
    return neutronv20.find_resourceid_by_name_or_id(client,
                                                    'network',
                                                    id_or_name)


def get_bgp_speaker_id(client, id_or_name):
    return neutronv20.find_resourceid_by_name_or_id(client,
                                                    'bgp_speaker',
                                                    id_or_name)


def validate_speaker_attributes(parsed_args):
    # Validate AS number
    validators.validate_int_range(parsed_args, 'local_as',
                                  MIN_AS_NUM, MAX_AS_NUM)


def add_common_arguments(parser):
    utils.add_boolean_argument(
        parser, '--advertise-floating-ip-host-routes',
        help=_('Whether to enable or disable the advertisement '
               'of floating-ip host routes by the BGP speaker. '
               'By default floating ip host routes will be '
               'advertised by the BGP speaker.'))
    utils.add_boolean_argument(
        parser, '--advertise-tenant-networks',
        help=_('Whether to enable or disable the advertisement '
               'of tenant network routes by the BGP speaker. '
               'By default tenant network routes will be '
               'advertised by the BGP speaker.'))


def args2body_common_arguments(body, parsed_args):
    neutronv20.update_dict(parsed_args, body,
                           ['name',
                            'advertise_floating_ip_host_routes',
                            'advertise_tenant_networks'])


class ListSpeakers(neutronv20.ListCommand):
    """List BGP speakers."""

    resource = 'bgp_speaker'
    list_columns = ['id', 'name', 'local_as', 'ip_version']
    pagination_support = True
    sorting_support = True


class ShowSpeaker(neutronv20.ShowCommand):
    """Show information of a given BGP speaker."""

    resource = 'bgp_speaker'


class CreateSpeaker(neutronv20.CreateCommand):
    """Create a BGP Speaker."""

    resource = 'bgp_speaker'

    def add_known_arguments(self, parser):
        parser.add_argument(
            'name',
            metavar='NAME',
            help=_('Name of the BGP speaker to create.'))
        parser.add_argument(
            '--local-as',
            metavar='LOCAL_AS',
            required=True,
            help=_('Local AS number. (Integer in [%(min_val)s, %(max_val)s] '
                   'is allowed.)') % {'min_val': MIN_AS_NUM,
                                      'max_val': MAX_AS_NUM})
        parser.add_argument(
            '--ip-version',
            type=int, choices=[4, 6],
            default=4,
            help=_('IP version for the BGP speaker (default is 4).'))
        add_common_arguments(parser)

    def args2body(self, parsed_args):
        body = {}
        validate_speaker_attributes(parsed_args)
        body['local_as'] = parsed_args.local_as
        body['ip_version'] = parsed_args.ip_version
        args2body_common_arguments(body, parsed_args)
        return {self.resource: body}


class UpdateSpeaker(neutronv20.UpdateCommand):
    """Update BGP Speaker's information."""

    resource = 'bgp_speaker'

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--name',
            help=_('Name of the BGP speaker to update.'))
        add_common_arguments(parser)

    def args2body(self, parsed_args):
        body = {}
        args2body_common_arguments(body, parsed_args)
        return {self.resource: body}


class DeleteSpeaker(neutronv20.DeleteCommand):
    """Delete a BGP speaker."""

    resource = 'bgp_speaker'


class AddPeerToSpeaker(neutronv20.NeutronCommand):
    """Add a peer to the BGP speaker."""

    def get_parser(self, prog_name):
        parser = super(AddPeerToSpeaker, self).get_parser(prog_name)
        parser.add_argument(
            'bgp_speaker',
            metavar='BGP_SPEAKER',
            help=_('ID or name of the BGP speaker.'))
        parser.add_argument(
            'bgp_peer',
            metavar='BGP_PEER',
            help=_('ID or name of the BGP peer to add.'))
        return parser

    def take_action(self, parsed_args):
        neutron_client = self.get_client()
        _speaker_id = get_bgp_speaker_id(neutron_client,
                                         parsed_args.bgp_speaker)
        _peer_id = bgp_peer.get_bgp_peer_id(neutron_client,
                                            parsed_args.bgp_peer)
        neutron_client.add_peer_to_bgp_speaker(_speaker_id,
                                               {'bgp_peer_id': _peer_id})
        print(_('Added BGP peer %(peer)s to BGP speaker %(speaker)s.') %
              {'peer': parsed_args.bgp_peer,
               'speaker': parsed_args.bgp_speaker},
              file=self.app.stdout)


class RemovePeerFromSpeaker(neutronv20.NeutronCommand):
    """Remove a peer from the BGP speaker."""

    def get_parser(self, prog_name):
        parser = super(RemovePeerFromSpeaker, self).get_parser(prog_name)
        parser.add_argument(
            'bgp_speaker',
            metavar='BGP_SPEAKER',
            help=_('ID or name of the BGP speaker.'))
        parser.add_argument(
            'bgp_peer',
            metavar='BGP_PEER',
            help=_('ID or name of the BGP peer to remove.'))
        return parser

    def take_action(self, parsed_args):
        neutron_client = self.get_client()
        _speaker_id = get_bgp_speaker_id(neutron_client,
                                         parsed_args.bgp_speaker)
        _peer_id = bgp_peer.get_bgp_peer_id(neutron_client,
                                            parsed_args.bgp_peer)
        neutron_client.remove_peer_from_bgp_speaker(_speaker_id,
                                                    {'bgp_peer_id': _peer_id})
        print(_('Removed BGP peer %(peer)s from BGP speaker %(speaker)s.') %
              {'peer': parsed_args.bgp_peer,
               'speaker': parsed_args.bgp_speaker},
              file=self.app.stdout)


class AddNetworkToSpeaker(neutronv20.NeutronCommand):
    """Add a network to the BGP speaker."""

    def get_parser(self, prog_name):
        parser = super(AddNetworkToSpeaker, self).get_parser(prog_name)
        parser.add_argument(
            'bgp_speaker',
            metavar='BGP_SPEAKER',
            help=_('ID or name of the BGP speaker.'))
        parser.add_argument(
            'network',
            metavar='NETWORK',
            help=_('ID or name of the network to add.'))
        return parser

    def take_action(self, parsed_args):
        neutron_client = self.get_client()
        _speaker_id = get_bgp_speaker_id(neutron_client,
                                         parsed_args.bgp_speaker)
        _net_id = get_network_id(neutron_client,
                                 parsed_args.network)
        neutron_client.add_network_to_bgp_speaker(_speaker_id,
                                                  {'network_id': _net_id})
        print(_('Added network %(net)s to BGP speaker %(speaker)s.') %
              {'net': parsed_args.network, 'speaker': parsed_args.bgp_speaker},
              file=self.app.stdout)


class RemoveNetworkFromSpeaker(neutronv20.NeutronCommand):
    """Remove a network from the BGP speaker."""

    def get_parser(self, prog_name):
        parser = super(RemoveNetworkFromSpeaker, self).get_parser(prog_name)
        parser.add_argument(
            'bgp_speaker',
            metavar='BGP_SPEAKER',
            help=_('ID or name of the BGP speaker.'))
        parser.add_argument(
            'network',
            metavar='NETWORK',
            help=_('ID or name of the network to remove.'))
        return parser

    def take_action(self, parsed_args):
        neutron_client = self.get_client()
        _speaker_id = get_bgp_speaker_id(neutron_client,
                                         parsed_args.bgp_speaker)
        _net_id = get_network_id(neutron_client,
                                 parsed_args.network)
        neutron_client.remove_network_from_bgp_speaker(_speaker_id,
                                                       {'network_id': _net_id})
        print(_('Removed network %(net)s from BGP speaker %(speaker)s.') %
              {'net': parsed_args.network, 'speaker': parsed_args.bgp_speaker},
              file=self.app.stdout)


class ListRoutesAdvertisedBySpeaker(neutronv20.ListCommand):
    """List routes advertised by a given BGP speaker."""

    list_columns = ['id', 'destination', 'next_hop']
    resource = 'advertised_route'
    pagination_support = True
    sorting_support = True

    def get_parser(self, prog_name):
        parser = super(ListRoutesAdvertisedBySpeaker,
                       self).get_parser(prog_name)
        parser.add_argument(
            'bgp_speaker',
            metavar='BGP_SPEAKER',
            help=_('ID or name of the BGP speaker.'))
        return parser

    def call_server(self, neutron_client, search_opts, parsed_args):
        _speaker_id = get_bgp_speaker_id(neutron_client,
                                         parsed_args.bgp_speaker)
        data = neutron_client.list_route_advertised_from_bgp_speaker(
            _speaker_id, **search_opts)
        return data



# Copyright 2017 FUJITSU LIMITED
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from unittest import mock

from osc_lib.cli import format_columns
from osc_lib.tests import utils as tests_utils

from neutronclient.osc import utils as osc_utils
from neutronclient.osc.v2.vpnaas import ipsec_site_connection
from neutronclient.tests.unit.osc.v2 import fakes as test_fakes
from neutronclient.tests.unit.osc.v2.vpnaas import common
from neutronclient.tests.unit.osc.v2.vpnaas import fakes


_ipsec_site_conn = fakes.IPsecSiteConnection().create_conn()
CONVERT_MAP = {
    'project': 'project_id',
    'ikepolicy': 'ikepolicy_id',
    'ipsecpolicy': 'ipsecpolicy_id',
    'vpnservice': 'vpnservice_id',
    'peer_endpoint_group': 'peer_ep_group_id',
    'local_endpoint_group': 'local_ep_group_id',
}


def _generate_data(ordered_dict=None, data=None):
    source = ordered_dict if ordered_dict else _ipsec_site_conn
    if data:
        source.update(data)
    return source


def _generate_req_and_res(verifylist):
    request = dict(verifylist)
    response = _ipsec_site_conn
    for key, val in verifylist:
        converted = CONVERT_MAP.get(key, key)
        del request[key]
        new_value = val
        request[converted] = new_value
        response[converted] = new_value
    return request, response


class TestIPsecSiteConn(test_fakes.TestNeutronClientOSCV2):

    def check_results(self, headers, data, exp_req, is_list=False):
        if is_list:
            req_body = {self.res_plural: list(exp_req)}
        else:
            req_body = exp_req
        self.mocked.assert_called_once_with(**req_body)
        self.assertEqual(self.ordered_headers, tuple(sorted(headers)))
        self.assertItemEqual(self.ordered_data, data)

    def setUp(self):
        super(TestIPsecSiteConn, self).setUp()

        def _mock_ipsec_site_conn(*args, **kwargs):
            return {'id': args[0]}

        self.networkclient.find_vpn_ipsec_site_connection.side_effect = \
            mock.Mock(side_effect=_mock_ipsec_site_conn)
        osc_utils.find_project = mock.Mock()
        osc_utils.find_project.id = _ipsec_site_conn['project_id']
        self.res = 'ipsec_site_connection'
        self.res_plural = 'ipsec_site_connections'
        self.resource = _ipsec_site_conn
        self.headers = (
            'ID',
            'Name',
            'Peer Address',
            'Authentication Algorithm',
            'Status',
            'Project',
            'Peer CIDRs',
            'VPN Service',
            'IPSec Policy',
            'IKE Policy',
            'MTU',
            'Initiator',
            'State',
            'Description',
            'Pre-shared Key',
            'Route Mode',
            'Local ID',
            'Peer ID',
            'Local Endpoint Group ID',
            'Peer Endpoint Group ID',
            'DPD',
        )
        self.data = _generate_data()
        self.ordered_headers = (
            'Authentication Algorithm',
            'DPD',
            'Description',
            'ID',
            'IKE Policy',
            'IPSec Policy',
            'Initiator',
            'Local Endpoint Group ID',
            'Local ID',
            'MTU',
            'Name',
            'Peer Address',
            'Peer CIDRs',
            'Peer Endpoint Group ID',
            'Peer ID',
            'Pre-shared Key',
            'Project',
            'Route Mode',
            'State',
            'Status',
            'VPN Service',
        )
        self.ordered_data = (
            _ipsec_site_conn['auth_mode'],
            _ipsec_site_conn['dpd'],
            _ipsec_site_conn['description'],
            _ipsec_site_conn['id'],
            _ipsec_site_conn['ikepolicy_id'],
            _ipsec_site_conn['ipsecpolicy_id'],
            _ipsec_site_conn['initiator'],
            _ipsec_site_conn['local_ep_group_id'],
            _ipsec_site_conn['local_id'],
            _ipsec_site_conn['mtu'],
            _ipsec_site_conn['name'],
            _ipsec_site_conn['peer_address'],
            format_columns.ListColumn(_ipsec_site_conn['peer_cidrs']),
            _ipsec_site_conn['peer_ep_group_id'],
            _ipsec_site_conn['peer_id'],
            _ipsec_site_conn['psk'],
            _ipsec_site_conn['project_id'],
            _ipsec_site_conn['route_mode'],
            _ipsec_site_conn['admin_state_up'],
            _ipsec_site_conn['status'],
            _ipsec_site_conn['vpnservice_id'],
        )


class TestCreateIPsecSiteConn(TestIPsecSiteConn, common.TestCreateVPNaaS):

    def setUp(self):
        super(TestCreateIPsecSiteConn, self).setUp()
        self.networkclient.create_vpn_ipsec_site_connection = mock.Mock(
            return_value=_ipsec_site_conn)
        self.mocked = self.networkclient.create_vpn_ipsec_site_connection
        self.cmd = ipsec_site_connection.CreateIPsecSiteConnection(
            self.app, self.namespace)

    def _update_expect_response(self, request, response):
        """Set expected request and response

        :param request
            A dictionary of request body(dict of verifylist)
        :param response
            A OrderedDict of request body
        """
        # Update response body
        self.networkclient.create_vpn_ipsec_site_connection.return_value = \
            response
        osc_utils.find_project.return_value.id = response['project_id']
        # Update response(finally returns 'data')
        self.data = _generate_data(ordered_dict=response)
        self.ordered_data = (
            response['auth_mode'],
            response['dpd'],
            response['description'],
            response['id'],
            response['ikepolicy_id'],
            response['ipsecpolicy_id'],
            response['initiator'],
            response['local_ep_group_id'],
            response['local_id'],
            response['mtu'],
            response['name'],
            response['peer_address'],
            format_columns.ListColumn(response['peer_cidrs']),
            response['peer_ep_group_id'],
            response['peer_id'],
            response['psk'],
            response['project_id'],
            response['route_mode'],
            response['admin_state_up'],
            response['status'],
            response['vpnservice_id'],
        )

    def _set_all_params(self, args={}):
        tenant_id = args.get('tenant_id') or 'my-tenant'
        name = args.get('name') or 'connection1'
        peer_address = args.get('peer_address') or '192.168.2.10'
        peer_id = args.get('peer_id') or '192.168.2.10'
        psk = args.get('psk') or 'abcd'
        mtu = args.get('mtu') or '1500'
        initiator = args.get('initiator') or 'bi-directional'
        vpnservice_id = args.get('vpnservice') or 'vpnservice_id'
        ikepolicy_id = args.get('ikepolicy') or 'ikepolicy_id'
        ipsecpolicy_id = args.get('ipsecpolicy') or 'ipsecpolicy_id'
        local_ep_group = args.get('local_ep_group_id') or 'local-epg'
        peer_ep_group = args.get('peer_ep_group_id') or 'peer-epg'
        description = args.get('description') or 'my-vpn-connection'

        arglist = [
            '--project', tenant_id,
            '--peer-address', peer_address,
            '--peer-id', peer_id,
            '--psk', psk,
            '--initiator', initiator,
            '--vpnservice', vpnservice_id,
            '--ikepolicy', ikepolicy_id,
            '--ipsecpolicy', ipsecpolicy_id,
            '--mtu', mtu,
            '--description', description,
            '--local-endpoint-group', local_ep_group,
            '--peer-endpoint-group', peer_ep_group,
            name,
        ]
        verifylist = [
            ('project', tenant_id),
            ('peer_address', peer_address),
            ('peer_id', peer_id),
            ('psk', psk),
            ('initiator', initiator),
            ('vpnservice', vpnservice_id),
            ('ikepolicy', ikepolicy_id),
            ('ipsecpolicy', ipsecpolicy_id),
            ('mtu', mtu),
            ('description', description),
            ('local_endpoint_group', local_ep_group),
            ('peer_endpoint_group', peer_ep_group),
            ('name', name),
        ]
        return arglist, verifylist

    def _test_create_with_all_params(self, args={}):
        arglist, verifylist = self._set_all_params(args)
        request, response = _generate_req_and_res(verifylist)

        def _mock_endpoint_group(*args, **kwargs):
            return {'id': args[0]}

        self.networkclient.find_vpn_endpoint_group.side_effect = mock.Mock(
            side_effect=_mock_endpoint_group)
        self.networkclient.find_vpn_service.side_effect = mock.Mock(
            side_effect=_mock_endpoint_group)
        self.networkclient.find_vpn_ike_policy.side_effect = mock.Mock(
            side_effect=_mock_endpoint_group)
        self.networkclient.find_vpn_ipsec_policy.side_effect = mock.Mock(
            side_effect=_mock_endpoint_group)
        self._update_expect_response(request, response)
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        headers, data = self.cmd.take_action(parsed_args)

        self.check_results(headers, data, request)

    def test_create_with_no_options(self):
        arglist = []
        verifylist = []

        self.assertRaises(tests_utils.ParserException, self.check_parser,
                          self.cmd, arglist, verifylist)

    def test_create_with_all_params(self):
        self._test_create_with_all_params()


class TestDeleteIPsecSiteConn(TestIPsecSiteConn, common.TestDeleteVPNaaS):

    def setUp(self):
        super(TestDeleteIPsecSiteConn, self).setUp()
        self.networkclient.delete_vpn_ipsec_site_connection = mock.Mock()
        self.mocked = self.networkclient.delete_vpn_ipsec_site_connection
        self.cmd = ipsec_site_connection.DeleteIPsecSiteConnection(
            self.app, self.namespace)


class TestListIPsecSiteConn(TestIPsecSiteConn):

    def setUp(self):
        super(TestListIPsecSiteConn, self).setUp()
        self.cmd = ipsec_site_connection.ListIPsecSiteConnection(
            self.app, self.namespace)

        self.short_header = (
            'ID',
            'Name',
            'Peer Address',
            'Authentication Algorithm',
            'Status',
        )

        self.short_data = (
            _ipsec_site_conn['id'],
            _ipsec_site_conn['name'],
            _ipsec_site_conn['peer_address'],
            _ipsec_site_conn['auth_mode'],
            _ipsec_site_conn['status'],
        )

        self.networkclient.vpn_ipsec_site_connections = mock.Mock(
            return_value=[_ipsec_site_conn])
        self.mocked = self.networkclient.vpn_ipsec_site_connections

    def test_list_with_long_option(self):
        arglist = ['--long']
        verifylist = [('long', True)]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        headers, data = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with()
        self.assertEqual(list(self.headers), headers)

    def test_list_with_no_option(self):
        arglist = []
        verifylist = []
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        headers, data = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with()
        self.assertEqual(list(self.short_header), headers)
        self.assertEqual([self.short_data], list(data))


class TestSetIPsecSiteConn(TestIPsecSiteConn, common.TestSetVPNaaS):

    def setUp(self):
        super(TestSetIPsecSiteConn, self).setUp()
        self.networkclient.update_vpn_ipsec_site_connection = mock.Mock(
            return_value=_ipsec_site_conn)
        self.mocked = self.networkclient.update_vpn_ipsec_site_connection
        self.cmd = ipsec_site_connection.SetIPsecSiteConnection(
            self.app, self.namespace)

    def test_set_ipsec_site_conn_with_peer_id(self):
        target = self.resource['id']
        peer_id = '192.168.3.10'
        arglist = [target, '--peer-id', peer_id]
        verifylist = [
            (self.res, target),
            ('peer_id', peer_id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(target, **{'peer_id': peer_id})
        self.assertIsNone(result)


class TestShowIPsecSiteConn(TestIPsecSiteConn, common.TestShowVPNaaS):

    def setUp(self):
        super(TestShowIPsecSiteConn, self).setUp()
        self.networkclient.get_vpn_ipsec_site_connection = mock.Mock(
            return_value=_ipsec_site_conn)
        self.mocked = self.networkclient.get_vpn_ipsec_site_connection
        self.cmd = ipsec_site_connection.ShowIPsecSiteConnection(
            self.app, self.namespace)



# Copyright (C) 2013 eNovance SAS <licensing@enovance.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from neutronclient._i18n import _
from neutronclient.common import utils
from neutronclient.neutron import v2_0 as neutronv20


class ListMeteringLabel(neutronv20.ListCommand):
    """List metering labels that belong to a given tenant."""

    resource = 'metering_label'
    list_columns = ['id', 'name', 'description', 'shared']
    pagination_support = True
    sorting_support = True


class ShowMeteringLabel(neutronv20.ShowCommand):
    """Show information of a given metering label."""

    resource = 'metering_label'
    allow_names = True


class CreateMeteringLabel(neutronv20.CreateCommand):
    """Create a metering label for a given tenant."""

    resource = 'metering_label'

    def add_known_arguments(self, parser):
        parser.add_argument(
            'name', metavar='NAME',
            help=_('Name of the metering label to be created.'))
        parser.add_argument(
            '--description',
            help=_('Description of the metering label to be created.'))
        parser.add_argument(
            '--shared',
            action='store_true',
            help=_('Set the label as shared.'))

    def args2body(self, parsed_args):
        body = {'name': parsed_args.name}
        neutronv20.update_dict(parsed_args, body,
                               ['tenant_id', 'description', 'shared'])
        return {'metering_label': body}


class DeleteMeteringLabel(neutronv20.DeleteCommand):
    """Delete a given metering label."""

    resource = 'metering_label'
    allow_names = True


class ListMeteringLabelRule(neutronv20.ListCommand):
    """List metering labels that belong to a given label."""

    resource = 'metering_label_rule'
    list_columns = ['id', 'excluded', 'direction', 'remote_ip_prefix']
    pagination_support = True
    sorting_support = True


class ShowMeteringLabelRule(neutronv20.ShowCommand):
    """Show information of a given metering label rule."""

    resource = 'metering_label_rule'
    allow_names = False


class CreateMeteringLabelRule(neutronv20.CreateCommand):
    """Create a metering label rule for a given label."""

    resource = 'metering_label_rule'

    def add_known_arguments(self, parser):
        parser.add_argument(
            'label_id', metavar='LABEL',
            help=_('ID or name of the label.'))
        parser.add_argument(
            'remote_ip_prefix', metavar='REMOTE_IP_PREFIX',
            help=_('CIDR to match on.'))
        parser.add_argument(
            '--direction',
            default='ingress', choices=['ingress', 'egress'],
            type=utils.convert_to_lowercase,
            help=_('Direction of traffic, default: ingress.'))
        parser.add_argument(
            '--excluded',
            action='store_true',
            help=_('Exclude this CIDR from the label, default: not excluded.'))

    def args2body(self, parsed_args):
        neutron_client = self.get_client()
        label_id = neutronv20.find_resourceid_by_name_or_id(
            neutron_client, 'metering_label', parsed_args.label_id)

        body = {'metering_label_id': label_id,
                'remote_ip_prefix': parsed_args.remote_ip_prefix}
        neutronv20.update_dict(parsed_args, body,
                               ['direction', 'excluded'])
        return {'metering_label_rule': body}


class DeleteMeteringLabelRule(neutronv20.DeleteCommand):
    """Delete a given metering label."""

    resource = 'metering_label_rule'
    allow_names = False



# Copyright 2017 FUJITSU LIMITED
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import testtools

from osc_lib import exceptions

from neutronclient.tests.unit.osc.v2 import fakes as test_fakes


class TestCreateVPNaaS(test_fakes.TestNeutronClientOSCV2):
    pass


class TestDeleteVPNaaS(test_fakes.TestNeutronClientOSCV2):

    def test_delete_with_one_resource(self):
        target = self.resource['id']

        def _mock_vpnaas(*args, **kwargs):
            return {'id': args[0]}

        self.networkclient.find_vpn_endpoint_group.side_effect = _mock_vpnaas
        self.networkclient.find_vpn_ipsec_site_connection.side_effect = \
            _mock_vpnaas
        self.networkclient.find_vpn_ike_policy.side_effect = _mock_vpnaas
        self.networkclient.find_vpn_ipsec_policy.side_effect = _mock_vpnaas
        self.networkclient.find_vpn_service.side_effect = _mock_vpnaas

        arglist = [target]
        verifylist = [(self.res, [target])]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(target)
        self.assertIsNone(result)

    def test_delete_with_multiple_resources(self):

        def _mock_vpnaas(*args, **kwargs):
            return {'id': args[0]}

        self.networkclient.find_vpn_endpoint_group.side_effect = _mock_vpnaas
        self.networkclient.find_vpn_ipsec_site_connection.side_effect = \
            _mock_vpnaas
        self.networkclient.find_vpn_ike_policy.side_effect = _mock_vpnaas
        self.networkclient.find_vpn_ipsec_policy.side_effect = _mock_vpnaas
        self.networkclient.find_vpn_service.side_effect = _mock_vpnaas

        target1 = 'target1'
        target2 = 'target2'
        arglist = [target1, target2]
        verifylist = [(self.res, [target1, target2])]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)
        self.assertIsNone(result)

        self.assertEqual(2, self.mocked.call_count)
        for idx, reference in enumerate([target1, target2]):
            actual = ''.join(self.mocked.call_args_list[idx][0])
            self.assertEqual(reference, actual)

    def test_delete_multiple_with_exception(self):
        target1 = 'target'
        arglist = [target1]
        verifylist = [(self.res, [target1])]

        self.networkclient.find_vpn_ipsec_site_connection.side_effect = [
            target1, exceptions.CommandError
        ]
        self.networkclient.find_vpn_endpoint_group.side_effect = [
            target1, exceptions.CommandError
        ]
        self.networkclient.find_vpn_ike_policy.side_effect = [
            target1, exceptions.CommandError
        ]
        self.networkclient.find_vpn_service.side_effect = [
            target1, exceptions.CommandError
        ]
        self.networkclient.find_vpn_ipsec_policy.side_effect = [
            target1, exceptions.CommandError
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        resource_name = self.res.replace('_', ' ')
        msg = "1 of 2 %s(s) failed to delete." % resource_name
        with testtools.ExpectedException(exceptions.CommandError) as e:
            self.cmd.take_action(parsed_args)
            self.assertEqual(msg, str(e))


class TestListVPNaaS(test_fakes.TestNeutronClientOSCV2):

    def test_list_with_no_option(self):
        arglist = []
        verifylist = []
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        headers, data = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with()
        self.assertEqual(list(self.list_headers), headers)
        self.assertEqual([self.list_data], list(data))

    def test_list_with_long_option(self):
        arglist = ['--long']
        verifylist = [('long', True)]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        headers, data = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with()
        self.assertEqual(list(self.headers), headers)
        self.assertEqual([self.data], list(data))


class TestSetVPNaaS(test_fakes.TestNeutronClientOSCV2):

    def test_set_name(self):
        target = self.resource['id']
        update = 'change'
        arglist = [target, '--name', update]
        verifylist = [
            (self.res, target),
            ('name', update),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(
            target, **{'name': update})
        self.assertIsNone(result)

    def test_set_description(self):
        target = self.resource['id']
        update = 'change-desc'
        arglist = [target, '--description', update]
        verifylist = [
            (self.res, target),
            ('description', update),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(
            target, **{'description': update})
        self.assertIsNone(result)


class TestShowVPNaaS(test_fakes.TestNeutronClientOSCV2):

    def test_show_filtered_by_id_or_name(self):
        target = self.resource['id']

        def _mock_vpnaas(*args, **kwargs):
            return {'id': args[0]}

        self.networkclient.find_vpn_endpoint_group.side_effect = _mock_vpnaas
        self.networkclient.find_vpn_ipsec_site_connection.side_effect = \
            _mock_vpnaas
        self.networkclient.find_vpn_ike_policy.side_effect = _mock_vpnaas
        self.networkclient.find_vpn_ipsec_policy.side_effect = _mock_vpnaas
        self.networkclient.find_vpn_service.side_effect = _mock_vpnaas

        arglist = [target]
        verifylist = [(self.res, target)]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        headers, data = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(target)
        self.assertEqual(self.ordered_headers, headers)
        self.assertItemEqual(self.ordered_data, data)



# Copyright 2016 Cisco Systems
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
import sys

from neutronclient._i18n import _
from neutronclient.neutron import v2_0 as neutronV20


class Purge(neutronV20.NeutronCommand):
    """Delete all resources that belong to a given tenant."""

    def _pluralize(self, string):
        return string + 's'

    def _get_resources(self, neutron_client, resource_types, tenant_id):
        resources = []
        for resource_type in resource_types:
            resources.append([])
            resource_type_plural = self._pluralize(resource_type)
            opts = {'fields': ['id', 'tenant_id']}
            if resource_type_plural == 'ports':
                opts['fields'].append('device_id')
                opts['fields'].append('device_owner')
            function = getattr(neutron_client, 'list_%s' %
                               resource_type_plural)
            if callable(function):
                returned_resources = function(**opts).get(resource_type_plural,
                                                          [])
                for resource in returned_resources:
                    if resource['tenant_id'] == tenant_id:
                        index = resource_types.index(resource_type)
                        resources[index].append(resource)
                        self.total_resources += 1
        return resources

    def _delete_resource(self, neutron_client, resource_type, resource):
        resource_id = resource['id']
        if resource_type == 'port':
            router_interface_owners = ['network:router_interface',
                                       'network:router_interface_distributed']
            if resource.get('device_owner', '') in router_interface_owners:
                body = {'port_id': resource_id}
                neutron_client.remove_interface_router(resource['device_id'],
                                                       body)
                return
        function = getattr(neutron_client, 'delete_%s' % resource_type)
        if callable(function):
            function(resource_id)

    def _purge_resources(self, neutron_client, resource_types,
                         tenant_resources):
        deleted = {}
        failed = {}
        failures = False
        for resources in tenant_resources:
            index = tenant_resources.index(resources)
            resource_type = resource_types[index]
            failed[resource_type] = 0
            deleted[resource_type] = 0
            for resource in resources:
                try:
                    self._delete_resource(neutron_client, resource_type,
                                          resource)
                    deleted[resource_type] += 1
                    self.deleted_resources += 1
                except Exception:
                    failures = True
                    failed[resource_type] += 1
                    self.total_resources -= 1
                percent_complete = 100
                if self.total_resources > 0:
                    percent_complete = (self.deleted_resources /
                                        float(self.total_resources)) * 100
                sys.stdout.write("\rPurging resources: %d%% complete." %
                                 percent_complete)
                sys.stdout.flush()
        return (deleted, failed, failures)

    def _build_message(self, deleted, failed, failures):
        msg = ''
        deleted_msg = []
        for resource, value in deleted.items():
            if value:
                if not msg:
                    msg = 'Deleted'
                if not value == 1:
                    resource = self._pluralize(resource)
                deleted_msg.append(" %d %s" % (value, resource))
        if deleted_msg:
            msg += ','.join(deleted_msg)

        failed_msg = []
        if failures:
            if msg:
                msg += '. '
            msg += 'The following resources could not be deleted:'
            for resource, value in failed.items():
                if value:
                    if not value == 1:
                        resource = self._pluralize(resource)
                    failed_msg.append(" %d %s" % (value, resource))
            msg += ','.join(failed_msg)

        if msg:
            msg += '.'
        else:
            msg = _('Tenant has no supported resources.')

        return msg

    def get_parser(self, prog_name):
        parser = super(Purge, self).get_parser(prog_name)
        parser.add_argument(
            'tenant', metavar='TENANT',
            help=_('ID of Tenant owning the resources to be deleted.'))
        return parser

    def take_action(self, parsed_args):
        neutron_client = self.get_client()

        self.any_failures = False

        # A list of the types of resources supported in the order in which
        # they should be deleted.
        resource_types = ['floatingip', 'port', 'router',
                          'network', 'security_group']

        deleted = {}
        failed = {}
        self.total_resources = 0
        self.deleted_resources = 0
        resources = self._get_resources(neutron_client, resource_types,
                                        parsed_args.tenant)
        deleted, failed, failures = self._purge_resources(neutron_client,
                                                          resource_types,
                                                          resources)
        print('\n%s' % self._build_message(deleted, failed, failures))



# Copyright 2015 Rackspace Hosting Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from stevedore import extension

from neutronclient.neutron import v2_0 as neutronV20


def _discover_via_entry_points():
    emgr = extension.ExtensionManager('neutronclient.extension',
                                      invoke_on_load=False)
    return ((ext.name, ext.plugin) for ext in emgr)


class NeutronClientExtension(neutronV20.NeutronCommand):
    pagination_support = False
    _formatters = {}
    sorting_support = False


class ClientExtensionShow(NeutronClientExtension, neutronV20.ShowCommand):
    def take_action(self, parsed_args):
        # NOTE(mdietz): Calls 'execute' to provide a consistent pattern
        #               for any implementers adding extensions with
        #               regard to any other extension verb.
        return self.execute(parsed_args)

    def execute(self, parsed_args):
        return super(ClientExtensionShow, self).take_action(parsed_args)


class ClientExtensionList(NeutronClientExtension, neutronV20.ListCommand):
    def take_action(self, parsed_args):
        # NOTE(mdietz): Calls 'execute' to provide a consistent pattern
        #               for any implementers adding extensions with
        #               regard to any other extension verb.
        return self.execute(parsed_args)

    def execute(self, parsed_args):
        return super(ClientExtensionList, self).take_action(parsed_args)


class ClientExtensionDelete(NeutronClientExtension, neutronV20.DeleteCommand):
    def take_action(self, parsed_args):
        # NOTE(mdietz): Calls 'execute' to provide a consistent pattern
        #               for any implementers adding extensions with
        #               regard to any other extension verb.
        return self.execute(parsed_args)

    def execute(self, parsed_args):
        return super(ClientExtensionDelete, self).take_action(parsed_args)


class ClientExtensionCreate(NeutronClientExtension, neutronV20.CreateCommand):
    def take_action(self, parsed_args):
        # NOTE(mdietz): Calls 'execute' to provide a consistent pattern
        #               for any implementers adding extensions with
        #               regard to any other extension verb.
        return self.execute(parsed_args)

    def execute(self, parsed_args):
        return super(ClientExtensionCreate, self).take_action(parsed_args)


class ClientExtensionUpdate(NeutronClientExtension, neutronV20.UpdateCommand):
    def take_action(self, parsed_args):
        # NOTE(mdietz): Calls 'execute' to provide a consistent pattern
        #               for any implementers adding extensions with
        #               regard to any other extension verb.
        return self.execute(parsed_args)

    def execute(self, parsed_args):
        return super(ClientExtensionUpdate, self).take_action(parsed_args)



# Copyright 2016 Comcast, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from neutronclient._i18n import _
from neutronclient.common import exceptions
from neutronclient.neutron import v2_0 as neutronv20
from neutronclient.neutron.v2_0.qos import rule as qos_rule


DSCP_MARKING_RESOURCE = 'dscp_marking_rule'
# DSCP DETAILS
# 0 - none  | 8 - cs1   | 10 - af11 | 12 - af12 | 14 - af13 |
# 16 - cs2  | 18 - af21 | 20 - af22 | 22 - af23 | 24 - cs3  |
# 26 - af31 | 28 - af32 | 30 - af33 | 32 - cs4  | 34 - af41 |
# 36 - af42 | 38 - af43 | 40 - cs5  | 46 - ef   | 48 - cs6  |
# 56 - cs7

DSCP_VALID_MARKS = [0, 8, 10, 12, 14, 16, 18, 20, 22, 24, 26, 28, 30, 32,
                    34, 36, 38, 40, 46, 48, 56]


def add_dscp_marking_arguments(parser):
    parser.add_argument(
        '--dscp-mark',
        required=True,
        type=str,
        help=_('DSCP mark: value can be 0, even numbers from 8-56, \
                excluding 42, 44, 50, 52, and 54.'))


def update_dscp_args2body(parsed_args, body):
    dscp_mark = parsed_args.dscp_mark
    if int(dscp_mark) not in DSCP_VALID_MARKS:
        raise exceptions.CommandError(_("DSCP mark: %s not supported. "
                                        "Please note value can either be 0 "
                                        "or any even number from 8-56 "
                                        "excluding 42, 44, 50, 52 and "
                                        "54.") % dscp_mark)
    neutronv20.update_dict(parsed_args, body,
                           ['dscp_mark'])


class CreateQoSDscpMarkingRule(qos_rule.QosRuleMixin,
                               neutronv20.CreateCommand):
    """Create a QoS DSCP marking rule."""

    resource = DSCP_MARKING_RESOURCE

    def add_known_arguments(self, parser):
        super(CreateQoSDscpMarkingRule, self).add_known_arguments(parser)
        add_dscp_marking_arguments(parser)

    def args2body(self, parsed_args):
        body = {}
        update_dscp_args2body(parsed_args, body)
        return {self.resource: body}


class ListQoSDscpMarkingRules(qos_rule.QosRuleMixin,
                              neutronv20.ListCommand):
    """List all QoS DSCP marking rules belonging to the specified policy."""

    _formatters = {}
    pagination_support = True
    sorting_support = True
    resource = DSCP_MARKING_RESOURCE


class ShowQoSDscpMarkingRule(qos_rule.QosRuleMixin,
                             neutronv20.ShowCommand):
    """Show information about the given qos dscp marking rule."""

    resource = DSCP_MARKING_RESOURCE
    allow_names = False


class UpdateQoSDscpMarkingRule(qos_rule.QosRuleMixin,
                               neutronv20.UpdateCommand):
    """Update the given QoS DSCP marking rule."""

    allow_names = False
    resource = DSCP_MARKING_RESOURCE

    def add_known_arguments(self, parser):
        super(UpdateQoSDscpMarkingRule, self).add_known_arguments(parser)
        add_dscp_marking_arguments(parser)

    def args2body(self, parsed_args):
        body = {}
        update_dscp_args2body(parsed_args, body)
        return {self.resource: body}


class DeleteQoSDscpMarkingRule(qos_rule.QosRuleMixin,
                               neutronv20.DeleteCommand):
    """Delete a given qos dscp marking rule."""

    allow_names = False
    resource = DSCP_MARKING_RESOURCE



# Copyright 2012 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import argparse

from neutronclient._i18n import _
from neutronclient.common import exceptions
from neutronclient.common import utils
from neutronclient.neutron import v2_0 as neutronV20
from neutronclient.neutron.v2_0 import availability_zone
from neutronclient.neutron.v2_0 import dns
from neutronclient.neutron.v2_0.qos import policy as qos_policy


def _format_subnets(network):
    try:
        return '\n'.join([' '.join([s['id'], s.get('cidr', '')])
                          for s in network['subnets']])
    except (TypeError, KeyError):
        return ''


def args2body_common(body, parsed_args):
    neutronV20.update_dict(parsed_args, body,
                           ['name', 'description'])


class ListNetwork(neutronV20.ListCommand):
    """List networks that belong to a given tenant."""

    # Length of a query filter on subnet id
    # id=<uuid>& (with len(uuid)=36)
    subnet_id_filter_len = 40
    # Length of a marker in pagination
    # &marker=<uuid> (with len(uuid)=36)
    marker_len = 44
    resource = 'network'
    _formatters = {'subnets': _format_subnets, }
    list_columns = ['id', 'name', 'subnets']
    pagination_support = True
    sorting_support = True

    filter_attrs = [
        'tenant_id',
        'name',
        'admin_state_up',
        {'name': 'status',
         'help': _("Filter %s according to their operation status."
                   "(For example: ACTIVE, ERROR etc)"),
         'boolean': False,
         'argparse_kwargs': {'type': utils.convert_to_uppercase}},
        {'name': 'shared',
         'help': _('Filter and list the networks which are shared.'),
         'boolean': True},
        {'name': 'router:external',
         'help': _('Filter and list the networks which are external.'),
         'boolean': True},
        {'name': 'tags',
         'help': _("Filter and list %s which has all given tags. "
                   "Multiple tags can be set like --tags <tag[,tag...]>"),
         'boolean': False,
         'argparse_kwargs': {'metavar': 'TAG'}},
        {'name': 'tags_any',
         'help': _("Filter and list %s which has any given tags. "
                   "Multiple tags can be set like --tags-any <tag[,tag...]>"),
         'boolean': False,
         'argparse_kwargs': {'metavar': 'TAG'}},
        {'name': 'not_tags',
         'help': _("Filter and list %s which does not have all given tags. "
                   "Multiple tags can be set like --not-tags <tag[,tag...]>"),
         'boolean': False,
         'argparse_kwargs': {'metavar': 'TAG'}},
        {'name': 'not_tags_any',
         'help': _("Filter and list %s which does not have any given tags. "
                   "Multiple tags can be set like --not-tags-any "
                   "<tag[,tag...]>"),
         'boolean': False,
         'argparse_kwargs': {'metavar': 'TAG'}},
    ]

    def extend_list(self, data, parsed_args):
        """Add subnet information to a network list."""
        neutron_client = self.get_client()
        search_opts = {'fields': ['id', 'cidr']}
        if self.pagination_support:
            page_size = parsed_args.page_size
            if page_size:
                search_opts.update({'limit': page_size})
        subnet_ids = []
        for n in data:
            if 'subnets' in n:
                subnet_ids.extend(n['subnets'])

        def _get_subnet_list(sub_ids):
            search_opts['id'] = sub_ids
            return neutron_client.list_subnets(
                **search_opts).get('subnets', [])

        try:
            subnets = _get_subnet_list(subnet_ids)
        except exceptions.RequestURITooLong as uri_len_exc:
            # The URI is too long because of too many subnet_id filters
            # Use the excess attribute of the exception to know how many
            # subnet_id filters can be inserted into a single request
            subnet_count = len(subnet_ids)
            max_size = ((self.subnet_id_filter_len * subnet_count) -
                        uri_len_exc.excess)
            if self.pagination_support:
                max_size -= self.marker_len
            chunk_size = max_size // self.subnet_id_filter_len
            subnets = []
            for i in range(0, subnet_count, chunk_size):
                subnets.extend(
                    _get_subnet_list(subnet_ids[i: i + chunk_size]))

        subnet_dict = dict([(s['id'], s) for s in subnets])
        for n in data:
            if 'subnets' in n:
                n['subnets'] = [(subnet_dict.get(s) or {"id": s})
                                for s in n['subnets']]


class ListExternalNetwork(ListNetwork):
    """List external networks that belong to a given tenant."""

    pagination_support = True
    sorting_support = True

    def retrieve_list(self, parsed_args):
        external = '--router:external=True'
        if external not in self.values_specs:
            self.values_specs.append('--router:external=True')
        return super(ListExternalNetwork, self).retrieve_list(parsed_args)


class ShowNetwork(neutronV20.ShowCommand):
    """Show information of a given network."""

    resource = 'network'


class CreateNetwork(neutronV20.CreateCommand, qos_policy.CreateQosPolicyMixin):
    """Create a network for a given tenant."""

    resource = 'network'

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--admin-state-down',
            dest='admin_state', action='store_false',
            help=_('Set admin state up to false.'))
        parser.add_argument(
            '--admin_state_down',
            dest='admin_state', action='store_false',
            help=argparse.SUPPRESS)
        parser.add_argument(
            '--shared',
            action='store_true',
            help=_('Set the network as shared.'),
            default=argparse.SUPPRESS)
        parser.add_argument(
            '--provider:network_type',
            metavar='<network_type>',
            help=_('The physical mechanism by which the virtual network'
                   ' is implemented.'))
        parser.add_argument(
            '--provider:physical_network',
            metavar='<physical_network_name>',
            help=_('Name of the physical network over which the virtual '
                   'network is implemented.'))
        parser.add_argument(
            '--provider:segmentation_id',
            metavar='<segmentation_id>',
            help=_('VLAN ID for VLAN networks or tunnel-id for GRE/VXLAN '
                   'networks.'))
        utils.add_boolean_argument(
            parser,
            '--vlan-transparent',
            default=argparse.SUPPRESS,
            help=_('Create a VLAN transparent network.'))
        parser.add_argument(
            'name', metavar='NAME',
            help=_('Name of the network to be created.'))
        parser.add_argument(
            '--description',
            help=_('Description of network.'))

        self.add_arguments_qos_policy(parser)
        availability_zone.add_az_hint_argument(parser, self.resource)
        dns.add_dns_argument_create(parser, self.resource, 'domain')

    def args2body(self, parsed_args):
        body = {'admin_state_up': parsed_args.admin_state}
        args2body_common(body, parsed_args)
        neutronV20.update_dict(parsed_args, body,
                               ['shared', 'tenant_id',
                                'vlan_transparent',
                                'provider:network_type',
                                'provider:physical_network',
                                'provider:segmentation_id',
                                'description'])
        self.args2body_qos_policy(parsed_args, body)
        availability_zone.args2body_az_hint(parsed_args, body)
        dns.args2body_dns_create(parsed_args, body, 'domain')
        return {'network': body}


class DeleteNetwork(neutronV20.DeleteCommand):
    """Delete a given network."""

    resource = 'network'


class UpdateNetwork(neutronV20.UpdateCommand, qos_policy.UpdateQosPolicyMixin):
    """Update network's information."""

    resource = 'network'

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--name',
            help=_('Name of the network.'))
        parser.add_argument(
            '--description',
            help=_('Description of this network.'))
        self.add_arguments_qos_policy(parser)
        dns.add_dns_argument_update(parser, self.resource, 'domain')

    def args2body(self, parsed_args):
        body = {}
        args2body_common(body, parsed_args)
        self.args2body_qos_policy(parsed_args, body)
        dns.args2body_dns_update(parsed_args, body, 'domain')
        return {'network': body}



# Copyright 2014 NEC Corporation
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from neutronclient.common import exceptions
from neutronclient.common import validators


class FakeParsedArgs(object):
    pass


class ValidatorTest(testtools.TestCase):

    def _test_validate_int(self, attr_val, attr_name='attr1',
                           min_value=1, max_value=10):
        obj = FakeParsedArgs()
        setattr(obj, attr_name, attr_val)
        ret = validators.validate_int_range(obj, attr_name,
                                            min_value, max_value)
        # Come here only if there is no exception.
        self.assertIsNone(ret)

    def _test_validate_int_error(self, attr_val, expected_msg,
                                 attr_name='attr1', expected_exc=None,
                                 min_value=1, max_value=10):
        if expected_exc is None:
            expected_exc = exceptions.CommandError
        e = self.assertRaises(expected_exc,
                              self._test_validate_int,
                              attr_val, attr_name, min_value, max_value)
        self.assertEqual(expected_msg, str(e))

    def test_validate_int_min_max(self):
        self._test_validate_int(1)
        self._test_validate_int(10)
        self._test_validate_int('1')
        self._test_validate_int('10')
        self._test_validate_int('0x0a')

        self._test_validate_int_error(
            0, 'attr1 "0" should be an integer [1:10].')
        self._test_validate_int_error(
            11, 'attr1 "11" should be an integer [1:10].')
        self._test_validate_int_error(
            '0x10', 'attr1 "0x10" should be an integer [1:10].')

    def test_validate_int_min_only(self):
        self._test_validate_int(1, max_value=None)
        self._test_validate_int(10, max_value=None)
        self._test_validate_int(11, max_value=None)
        self._test_validate_int_error(
            0, 'attr1 "0" should be an integer greater than or equal to 1.',
            max_value=None)

    def test_validate_int_max_only(self):
        self._test_validate_int(0, min_value=None)
        self._test_validate_int(1, min_value=None)
        self._test_validate_int(10, min_value=None)
        self._test_validate_int_error(
            11, 'attr1 "11" should be an integer smaller than or equal to 10.',
            min_value=None)

    def test_validate_int_no_limit(self):
        self._test_validate_int(0, min_value=None, max_value=None)
        self._test_validate_int(1, min_value=None, max_value=None)
        self._test_validate_int(10, min_value=None, max_value=None)
        self._test_validate_int(11, min_value=None, max_value=None)
        self._test_validate_int_error(
            'abc', 'attr1 "abc" should be an integer.',
            min_value=None, max_value=None)

    def _test_validate_subnet(self, attr_val, attr_name='attr1'):
        obj = FakeParsedArgs()
        setattr(obj, attr_name, attr_val)
        ret = validators.validate_ip_subnet(obj, attr_name)
        # Come here only if there is no exception.
        self.assertIsNone(ret)

    def test_validate_ip_subnet(self):
        self._test_validate_subnet('192.168.2.0/24')
        self._test_validate_subnet('192.168.2.3/20')
        self._test_validate_subnet('192.168.2.1')

        e = self.assertRaises(exceptions.CommandError,
                              self._test_validate_subnet,
                              '192.168.2.256')
        self.assertEqual('attr1 "192.168.2.256" is not a valid CIDR.', str(e))



# Copyright 2012 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import argparse

from neutronclient._i18n import _
from neutronclient.neutron import v2_0 as neutronV20
from neutronclient.neutron.v2_0 import dns


class ListFloatingIP(neutronV20.ListCommand):
    """List floating IPs that belong to a given tenant."""

    resource = 'floatingip'
    list_columns = ['id', 'fixed_ip_address', 'floating_ip_address',
                    'port_id']
    pagination_support = True
    sorting_support = True


class ShowFloatingIP(neutronV20.ShowCommand):
    """Show information of a given floating IP."""

    resource = 'floatingip'
    allow_names = False


class CreateFloatingIP(neutronV20.CreateCommand):
    """Create a floating IP for a given tenant."""

    resource = 'floatingip'

    def add_known_arguments(self, parser):
        parser.add_argument(
            'floating_network_id', metavar='FLOATING_NETWORK',
            help=_('ID or name of the network from which '
                   'the floating IP is allocated.'))
        parser.add_argument(
            '--description',
            help=_('Description of the floating IP.'))
        parser.add_argument(
            '--port-id',
            help=_('ID of the port to be associated with the floating IP.'))
        parser.add_argument(
            '--port_id',
            help=argparse.SUPPRESS)
        parser.add_argument(
            '--fixed-ip-address',
            help=_('IP address on the port (only required if port has '
                   'multiple IPs).'))
        parser.add_argument(
            '--fixed_ip_address',
            help=argparse.SUPPRESS)
        parser.add_argument(
            '--floating-ip-address',
            help=_('IP address of the floating IP'))
        parser.add_argument(
            '--subnet',
            dest='subnet_id',
            help=_('Subnet ID on which you want to create the floating IP.'))
        dns.add_dns_argument_create(parser, self.resource, 'domain')
        dns.add_dns_argument_create(parser, self.resource, 'name')

    def args2body(self, parsed_args):
        _network_id = neutronV20.find_resourceid_by_name_or_id(
            self.get_client(), 'network', parsed_args.floating_network_id)
        body = {'floating_network_id': _network_id}
        neutronV20.update_dict(parsed_args, body,
                               ['port_id', 'tenant_id',
                                'fixed_ip_address', 'description',
                                'floating_ip_address', 'subnet_id'])
        dns.args2body_dns_create(parsed_args, body, 'domain')
        dns.args2body_dns_create(parsed_args, body, 'name')
        return {self.resource: body}


class DeleteFloatingIP(neutronV20.DeleteCommand):
    """Delete a given floating IP."""

    resource = 'floatingip'
    allow_names = False


class AssociateFloatingIP(neutronV20.NeutronCommand):
    """Create a mapping between a floating IP and a fixed IP."""

    resource = 'floatingip'

    def get_parser(self, prog_name):
        parser = super(AssociateFloatingIP, self).get_parser(prog_name)
        parser.add_argument(
            'floatingip_id', metavar='FLOATINGIP_ID',
            help=_('ID of the floating IP to associate.'))
        parser.add_argument(
            'port_id', metavar='PORT',
            help=_('ID or name of the port to be associated with the '
                   'floating IP.'))
        parser.add_argument(
            '--fixed-ip-address',
            help=_('IP address on the port (only required if port has '
                   'multiple IPs).'))
        parser.add_argument(
            '--fixed_ip_address',
            help=argparse.SUPPRESS)
        return parser

    def take_action(self, parsed_args):
        neutron_client = self.get_client()
        update_dict = {}
        neutronV20.update_dict(parsed_args, update_dict,
                               ['port_id', 'fixed_ip_address'])
        neutron_client.update_floatingip(parsed_args.floatingip_id,
                                         {'floatingip': update_dict})
        print(_('Associated floating IP %s') % parsed_args.floatingip_id,
              file=self.app.stdout)


class DisassociateFloatingIP(neutronV20.NeutronCommand):
    """Remove a mapping from a floating IP to a fixed IP."""

    resource = 'floatingip'

    def get_parser(self, prog_name):
        parser = super(DisassociateFloatingIP, self).get_parser(prog_name)
        parser.add_argument(
            'floatingip_id', metavar='FLOATINGIP_ID',
            help=_('ID of the floating IP to disassociate.'))
        return parser

    def take_action(self, parsed_args):
        neutron_client = self.get_client()
        neutron_client.update_floatingip(parsed_args.floatingip_id,
                                         {'floatingip': {'port_id': None}})
        print(_('Disassociated floating IP %s') % parsed_args.floatingip_id,
              file=self.app.stdout)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from neutronclient._i18n import _
from neutronclient.neutron import v2_0 as neutronv20


def add_az_hint_argument(parser, resource):
    parser.add_argument(
        '--availability-zone-hint', metavar='AVAILABILITY_ZONE',
        action='append', dest='availability_zone_hints',
        help=_('Availability Zone for the %s '
               '(requires availability zone extension, '
               'this option can be repeated).') % resource)


def args2body_az_hint(parsed_args, resource):
    if parsed_args.availability_zone_hints:
        resource['availability_zone_hints'] = (
            parsed_args.availability_zone_hints)


class ListAvailabilityZone(neutronv20.ListCommand):
    """List availability zones."""

    resource = 'availability_zone'
    list_columns = ['name', 'resource', 'state']
    pagination_support = True
    sorting_support = True



# Copyright (c) 2017 Huawei Technologies India Pvt.Limited.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import argparse
import logging

from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils
from osc_lib.utils import columns as column_util

from neutronclient._i18n import _
from neutronclient.common import exceptions as nc_exc

LOG = logging.getLogger(__name__)

resource = 'flow_classifier'

_attr_map = (
    ('id', 'ID', column_util.LIST_BOTH),
    ('name', 'Name', column_util.LIST_BOTH),
    ('summary', 'Summary', column_util.LIST_SHORT_ONLY),
    ('protocol', 'Protocol', column_util.LIST_LONG_ONLY),
    ('ethertype', 'Ethertype', column_util.LIST_LONG_ONLY),
    ('source_ip_prefix', 'Source IP',
     column_util.LIST_LONG_ONLY),
    ('destination_ip_prefix', 'Destination IP',
     column_util.LIST_LONG_ONLY),
    ('logical_source_port', 'Logical Source Port',
     column_util.LIST_LONG_ONLY),
    ('logical_destination_port', 'Logical Destination Port',
     column_util.LIST_LONG_ONLY),
    ('source_port_range_min', 'Source Port Range Min',
     column_util.LIST_LONG_ONLY),
    ('source_port_range_max', 'Source Port Range Max',
     column_util.LIST_LONG_ONLY),
    ('destination_port_range_min', 'Destination Port Range Min',
     column_util.LIST_LONG_ONLY),
    ('destination_port_range_max', 'Destination Port Range Max',
     column_util.LIST_LONG_ONLY),
    ('l7_parameters', 'L7 Parameters', column_util.LIST_LONG_ONLY),
    ('description', 'Description', column_util.LIST_LONG_ONLY),
    ('project_id', 'Project', column_util.LIST_LONG_ONLY),
)

_attr_map_dict = {
    'id': 'ID',
    'name': 'Name',
    'description': 'Description',
    'summary': 'Summary',
    'protocol': 'Protocol',
    'ethertype': 'Ethertype',
    'source_ip_prefix': 'Source IP',
    'destination_ip_prefix': 'Destination IP',
    'logical_source_port': 'Logical Source Port',
    'logical_destination_port': 'Logical Destination Port',
    'source_port_range_min': 'Source Port Range Min',
    'source_port_range_max': 'Source Port Range Max',
    'destination_port_range_min': 'Destination Port Range Min',
    'destination_port_range_max': 'Destination Port Range Max',
    'l7_parameters': 'L7 Parameters',
    'tenant_id': 'Project',
    'project_id': 'Project',
}


class CreateSfcFlowClassifier(command.ShowOne):
    _description = _("Create a flow classifier")

    def get_parser(self, prog_name):
        parser = super(CreateSfcFlowClassifier, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('Name of the flow classifier'))
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Description for the flow classifier'))
        parser.add_argument(
            '--protocol',
            metavar='<protocol>',
            help=_('IP protocol name. Protocol name should be as per '
                   'IANA standard.'))
        parser.add_argument(
            '--ethertype',
            metavar='{IPv4,IPv6}',
            default='IPv4', choices=['IPv4', 'IPv6'],
            help=_('L2 ethertype, default is IPv4'))
        parser.add_argument(
            '--source-port',
            metavar='<min-port>:<max-port>',
            help=_('Source protocol port (allowed range [1,65535]. Must be '
                   'specified as a:b, where a=min-port and b=max-port) '
                   'in the allowed range.'))
        parser.add_argument(
            '--destination-port',
            metavar='<min-port>:<max-port>',
            help=_('Destination protocol port (allowed range [1,65535]. Must '
                   'be specified as a:b, where a=min-port and b=max-port) '
                   'in the allowed range.'))
        parser.add_argument(
            '--source-ip-prefix',
            metavar='<source-ip-prefix>',
            help=_('Source IP address in CIDR notation'))
        parser.add_argument(
            '--destination-ip-prefix',
            metavar='<destination-ip-prefix>',
            help=_('Destination IP address in CIDR notation'))
        parser.add_argument(
            '--logical-source-port',
            metavar='<logical-source-port>',
            help=_('Neutron source port (name or ID)'))
        parser.add_argument(
            '--logical-destination-port',
            metavar='<logical-destination-port>',
            help=_('Neutron destination port (name or ID)'))
        parser.add_argument(
            '--l7-parameters',
            help=_('Dictionary of L7 parameters. Currently, no value is '
                   'supported for this option.'))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.network
        attrs = _get_common_attrs(self.app.client_manager, parsed_args)
        obj = client.create_sfc_flow_classifier(**attrs)
        display_columns, columns = utils.get_osc_show_columns_for_sdk_resource(
            obj, _attr_map_dict, ['location', 'tenant_id', 'summary'])
        data = utils.get_dict_properties(obj, columns)
        return display_columns, data


class DeleteSfcFlowClassifier(command.Command):
    _description = _("Delete a given flow classifier")

    def get_parser(self, prog_name):
        parser = super(DeleteSfcFlowClassifier, self).get_parser(prog_name)
        parser.add_argument(
            'flow_classifier',
            metavar='<flow-classifier>',
            nargs='+',
            help=_("Flow classifier(s) to delete (name or ID)")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.network
        result = 0
        for fcl in parsed_args.flow_classifier:
            try:
                fc_id = client.find_sfc_flow_classifier(
                    fcl, ignore_missing=False)['id']
                client.delete_sfc_flow_classifier(fc_id)
            except Exception as e:
                result += 1
                LOG.error(_("Failed to delete flow classifier with name "
                            "or ID '%(fc)s': %(e)s"), {'fc': fcl, 'e': e})
        if result > 0:
            total = len(parsed_args.flow_classifier)
            msg = (_("%(result)s of %(total)s flow classifier(s) "
                     "failed to delete.") % {'result': result, 'total': total})
            raise exceptions.CommandError(msg)


class ListSfcFlowClassifier(command.Lister):
    _description = _("List flow classifiers")

    def get_parser(self, prog_name):
        parser = super(ListSfcFlowClassifier, self).get_parser(prog_name)
        parser.add_argument(
            '--long',
            action='store_true',
            help=_("List additional fields in output")
        )
        return parser

    def extend_list(self, data, parsed_args):
        ext_data = []
        for d in data:
            val = []
            protocol = d['protocol'].upper() if d['protocol'] else 'any'
            val.append('protocol: ' + protocol)
            val.append(self._get_protocol_port_details(d, 'source'))
            val.append(self._get_protocol_port_details(d, 'destination'))
            if 'logical_source_port' in d:
                val.append('neutron_source_port: ' +
                           str(d['logical_source_port']))

            if 'logical_destination_port' in d:
                val.append('neutron_destination_port: ' +
                           str(d['logical_destination_port']))

            if 'l7_parameters' in d:
                l7_param = 'l7_parameters: {%s}' % ','.join(d['l7_parameters'])
                val.append(l7_param)
            d['summary'] = ',\n'.join(val)
            ext_data.append(d)
        return ext_data

    def _get_protocol_port_details(self, data, val):
        type_ip_prefix = val + '_ip_prefix'
        ip_prefix = data.get(type_ip_prefix)
        if not ip_prefix:
            ip_prefix = 'any'
        min_port = data.get(val + '_port_range_min')
        if min_port is None:
            min_port = 'any'
        max_port = data.get(val + '_port_range_max')
        if max_port is None:
            max_port = 'any'
        return '%s[port]: %s[%s:%s]' % (
            val, ip_prefix, min_port, max_port)

    def take_action(self, parsed_args):
        client = self.app.client_manager.network
        obj = client.sfc_flow_classifiers()
        obj_extend = self.extend_list(obj, parsed_args)
        headers, columns = column_util.get_column_definitions(
            _attr_map, long_listing=parsed_args.long)
        return (headers, (utils.get_dict_properties(
            s, columns) for s in obj_extend))


class SetSfcFlowClassifier(command.Command):
    _description = _("Set flow classifier properties")

    def get_parser(self, prog_name):
        parser = super(SetSfcFlowClassifier, self).get_parser(prog_name)
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Name of the flow classifier'))
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Description for the flow classifier'))
        parser.add_argument(
            'flow_classifier',
            metavar='<flow-classifier>',
            help=_("Flow classifier to modify (name or ID)")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.network
        fc_id = client.find_sfc_flow_classifier(parsed_args.flow_classifier,
                                                ignore_missing=False)['id']
        attrs = _get_common_attrs(self.app.client_manager, parsed_args,
                                  is_create=False)
        try:
            client.update_sfc_flow_classifier(fc_id, **attrs)
        except Exception as e:
            msg = (_("Failed to update flow classifier '%(fc)s': %(e)s")
                   % {'fc': parsed_args.flow_classifier, 'e': e})
            raise exceptions.CommandError(msg)


class ShowSfcFlowClassifier(command.ShowOne):
    _description = _("Display flow classifier details")

    def get_parser(self, prog_name):
        parser = super(ShowSfcFlowClassifier, self).get_parser(prog_name)
        parser.add_argument(
            'flow_classifier',
            metavar='<flow-classifier>',
            help=_("Flow classifier to display (name or ID)")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.network
        fc_id = client.find_sfc_flow_classifier(parsed_args.flow_classifier,
                                                ignore_missing=False)['id']
        obj = client.get_sfc_flow_classifier(fc_id)
        display_columns, columns = utils.get_osc_show_columns_for_sdk_resource(
            obj, _attr_map_dict, ['location', 'tenant_id', 'summary'])
        data = utils.get_dict_properties(obj, columns)
        return display_columns, data


def _get_common_attrs(client_manager, parsed_args, is_create=True):
    attrs = {}
    if parsed_args.name is not None:
        attrs['name'] = parsed_args.name
    if parsed_args.description is not None:
        attrs['description'] = parsed_args.description
    if is_create:
        _get_attrs(client_manager, attrs, parsed_args)
    return attrs


def _get_attrs(client_manager, attrs, parsed_args):
    if parsed_args.protocol is not None:
        attrs['protocol'] = parsed_args.protocol
    if parsed_args.ethertype:
        attrs['ethertype'] = parsed_args.ethertype
    if parsed_args.source_ip_prefix is not None:
        attrs['source_ip_prefix'] = parsed_args.source_ip_prefix
    if parsed_args.destination_ip_prefix is not None:
        attrs['destination_ip_prefix'] = parsed_args.destination_ip_prefix
    if parsed_args.logical_source_port is not None:
        attrs['logical_source_port'] = client_manager.network.find_port(
            parsed_args.logical_source_port, ignore_missing=False
        )['id']
    if parsed_args.logical_destination_port is not None:
        attrs['logical_destination_port'] = client_manager.network.find_port(
            parsed_args.logical_destination_port, ignore_missing=False
        )['id']
    if parsed_args.source_port is not None:
        _fill_protocol_port_info(attrs, 'source',
                                        parsed_args.source_port)
    if parsed_args.destination_port is not None:
        _fill_protocol_port_info(attrs, 'destination',
                                        parsed_args.destination_port)
    if parsed_args.l7_parameters is not None:
        attrs['l7_parameters'] = parsed_args.l7_parameters


def _fill_protocol_port_info(attrs, port_type, port_val):
    min_port, sep, max_port = port_val.partition(":")
    if not min_port:
        msg = ("Invalid port value '%s', expected format is "
               "min-port:max-port or min-port.")
        raise argparse.ArgumentTypeError(msg % port_val)
    if not max_port:
        max_port = min_port
    try:
        attrs[port_type + '_port_range_min'] = int(min_port)
        attrs[port_type + '_port_range_max'] = int(max_port)
    except ValueError:
        message = (_("Protocol port value %s must be an integer "
                     "or integer:integer.") % port_val)
        raise nc_exc.CommandError(message=message)



#    (c) Copyright 2015 Cisco Systems, Inc.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from neutronclient._i18n import _
from neutronclient.neutron import v2_0 as neutronv20


def add_known_endpoint_group_arguments(parser, is_create=True):
    parser.add_argument(
        '--name',
        help=_('Set a name for the endpoint group.'))
    parser.add_argument(
        '--description',
        help=_('Set a description for the endpoint group.'))
    if is_create:
        parser.add_argument(
            '--type',
            required=is_create,
            help=_('Type of endpoints in group (e.g. subnet, cidr, vlan).'))
        parser.add_argument(
            '--value',
            action='append', dest='endpoints',
            required=is_create,
            help=_('Endpoint(s) for the group. Must all be of the same type.'))


class ListEndpointGroup(neutronv20.ListCommand):
    """List VPN endpoint groups that belong to a given tenant."""

    resource = 'endpoint_group'
    list_columns = ['id', 'name', 'type', 'endpoints']
    _formatters = {}
    pagination_support = True
    sorting_support = True


class ShowEndpointGroup(neutronv20.ShowCommand):
    """Show a specific VPN endpoint group."""

    resource = 'endpoint_group'


class CreateEndpointGroup(neutronv20.CreateCommand):
    """Create a VPN endpoint group."""
    resource = 'endpoint_group'

    def add_known_arguments(self, parser):
        add_known_endpoint_group_arguments(parser)

    def subnet_name2id(self, endpoint):
        return neutronv20.find_resourceid_by_name_or_id(self.get_client(),
                                                        'subnet', endpoint)

    def args2body(self, parsed_args):
        if parsed_args.type == 'subnet':
            endpoints = [self.subnet_name2id(ep)
                         for ep in parsed_args.endpoints]
        else:
            endpoints = parsed_args.endpoints

        body = {'endpoints': endpoints}

        neutronv20.update_dict(parsed_args, body,
                               ['name', 'description',
                                'tenant_id', 'type'])
        return {self.resource: body}


class UpdateEndpointGroup(neutronv20.UpdateCommand):
    """Update a given VPN endpoint group."""

    resource = 'endpoint_group'

    def add_known_arguments(self, parser):
        add_known_endpoint_group_arguments(parser, is_create=False)

    def args2body(self, parsed_args):
        body = {}
        neutronv20.update_dict(parsed_args, body,
                               ['name', 'description'])
        return {self.resource: body}


class DeleteEndpointGroup(neutronv20.DeleteCommand):
    """Delete a given VPN endpoint group."""

    resource = 'endpoint_group'



#    Copyright 2017 FUJITSU LIMITED
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from osc_lib.cli import format_columns
from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils
from osc_lib.utils import columns as column_util
from oslo_log import log as logging

from neutronclient._i18n import _
from neutronclient.common import utils as nc_utils
from neutronclient.osc import utils as osc_utils
from neutronclient.osc.v2.vpnaas import utils as vpn_utils


LOG = logging.getLogger(__name__)


_formatters = {
    'peer_cidrs': format_columns.ListColumn
}


_attr_map = (
    ('id', 'ID', column_util.LIST_BOTH),
    ('name', 'Name', column_util.LIST_BOTH),
    ('peer_address', 'Peer Address', column_util.LIST_BOTH),
    ('auth_mode', 'Authentication Algorithm', column_util.LIST_BOTH),
    ('status', 'Status', column_util.LIST_BOTH),
    ('project_id', 'Project', column_util.LIST_LONG_ONLY),
    ('peer_cidrs', 'Peer CIDRs', column_util.LIST_LONG_ONLY),
    ('vpnservice_id', 'VPN Service', column_util.LIST_LONG_ONLY),
    ('ipsecpolicy_id', 'IPSec Policy', column_util.LIST_LONG_ONLY),
    ('ikepolicy_id', 'IKE Policy', column_util.LIST_LONG_ONLY),
    ('mtu', 'MTU', column_util.LIST_LONG_ONLY),
    ('initiator', 'Initiator', column_util.LIST_LONG_ONLY),
    ('is_admin_state_up', 'State', column_util.LIST_LONG_ONLY),
    ('description', 'Description', column_util.LIST_LONG_ONLY),
    ('psk', 'Pre-shared Key', column_util.LIST_LONG_ONLY),
    ('route_mode', 'Route Mode', column_util.LIST_LONG_ONLY),
    ('local_id', 'Local ID', column_util.LIST_LONG_ONLY),
    ('peer_id', 'Peer ID', column_util.LIST_LONG_ONLY),
    ('local_ep_group_id', 'Local Endpoint Group ID',
     column_util.LIST_LONG_ONLY),
    ('peer_ep_group_id', 'Peer Endpoint Group ID', column_util.LIST_LONG_ONLY),
    ('dpd', 'DPD', column_util.LIST_LONG_ONLY),
)

_attr_map_dict = {
    'id': 'ID',
    'name': 'Name',
    'peer_address': 'Peer Address',
    'auth_mode': 'Authentication Algorithm',
    'status': 'Status',
    'peer_cidrs': 'Peer CIDRs',
    'vpnservice_id': 'VPN Service',
    'ipsecpolicy_id': 'IPSec Policy',
    'ikepolicy_id': 'IKE Policy',
    'mtu': 'MTU',
    'initiator': 'Initiator',
    'is_admin_state_up': 'State',
    'psk': 'Pre-shared Key',
    'route_mode': 'Route Mode',
    'local_id': 'Local ID',
    'peer_id': 'Peer ID',
    'local_ep_group_id': 'Local Endpoint Group ID',
    'peer_ep_group_id': 'Peer Endpoint Group ID',
    'description': 'Description',
    'project_id': 'Project',
    'dpd': 'DPD',
}


def _convert_to_lowercase(string):
    return string.lower()


def _get_common_parser(parser, is_create=True):
    parser.add_argument(
        '--description',
        metavar='<description>',
        help=_('Description for the connection'))
    parser.add_argument(
        '--dpd',
        metavar="action=ACTION,interval=INTERVAL,timeout=TIMEOUT",
        type=nc_utils.str2dict_type(
            optional_keys=['action', 'interval', 'timeout']),
        help=vpn_utils.dpd_help("IPsec connection"))
    parser.add_argument(
        '--mtu',
        help=_('MTU size for the connection'))
    parser.add_argument(
        '--initiator',
        choices=['bi-directional', 'response-only'],
        type=_convert_to_lowercase,
        help=_('Initiator state'))
    peer_group = parser.add_mutually_exclusive_group()
    peer_group.add_argument(
        '--peer-cidr',
        dest='peer_cidrs',
        help=_('Remote subnet(s) in CIDR format. '
               'Cannot be specified when using endpoint groups. Only '
               'applicable, if subnet provided for VPN service.')
    )
    peer_group.add_argument(
        '--local-endpoint-group',
        help=_('Local endpoint group (name or ID) with subnet(s) '
               'for IPsec connection')
    )
    parser.add_argument(
        '--peer-endpoint-group',
        help=_('Peer endpoint group (name or ID) with CIDR(s) for '
               'IPSec connection'))
    admin_group = parser.add_mutually_exclusive_group()
    admin_group.add_argument(
        '--enable',
        action='store_true',
        help=_("Enable IPSec site connection")
    )
    admin_group.add_argument(
        '--disable',
        action='store_true',
        help=_("Disable IPSec site connection")
    )
    parser.add_argument(
        '--local-id',
        help=_('An ID to be used instead of the external IP '
               'address for a virtual router'))
    return parser


def _get_common_attrs(client_manager, parsed_args, is_create=True):
    attrs = {}
    if is_create:
        if 'project' in parsed_args and parsed_args.project is not None:
            attrs['project_id'] = osc_utils.find_project(
                client_manager.identity,
                parsed_args.project,
                parsed_args.project_domain,
            ).id
    if parsed_args.description:
        attrs['description'] = str(parsed_args.description)
    if parsed_args.mtu:
        attrs['mtu'] = parsed_args.mtu
    if parsed_args.enable:
        attrs['admin_state_up'] = True
    if parsed_args.disable:
        attrs['admin_state_up'] = False
    if parsed_args.initiator:
        attrs['initiator'] = parsed_args.initiator
    if parsed_args.dpd:
        vpn_utils.validate_dpd_dict(parsed_args.dpd)
        attrs['dpd'] = parsed_args.dpd
    if parsed_args.local_endpoint_group:
        _local_epg = client_manager.network.find_vpn_endpoint_group(
            parsed_args.local_endpoint_group)['id']
        attrs['local_ep_group_id'] = _local_epg
    if parsed_args.peer_endpoint_group:
        _peer_epg = client_manager.network.find_vpn_endpoint_group(
            parsed_args.peer_endpoint_group)['id']
        attrs['peer_ep_group_id'] = _peer_epg
    if parsed_args.peer_cidrs:
        attrs['peer_cidrs'] = parsed_args.peer_cidrs
    if parsed_args.local_id:
        attrs['local_id'] = parsed_args.local_id
    return attrs


class CreateIPsecSiteConnection(command.ShowOne):
    _description = _("Create an IPsec site connection")

    def get_parser(self, prog_name):
        parser = super(CreateIPsecSiteConnection, self).get_parser(prog_name)
        _get_common_parser(parser)
        parser.add_argument(
            '--peer-id',
            required=True,
            help=_('Peer router identity for authentication. Can be '
                   'IPv4/IPv6 address, e-mail address, key id, or FQDN'))
        parser.add_argument(
            '--peer-address',
            required=True,
            help=_('Peer gateway public IPv4/IPv6 address or FQDN'))
        parser.add_argument(
            '--psk',
            required=True,
            help=_('Pre-shared key string.'))
        parser.add_argument(
            '--vpnservice',
            metavar='VPNSERVICE',
            required=True,
            help=_('VPN service instance associated with this '
                   'connection (name or ID)'))
        parser.add_argument(
            '--ikepolicy',
            metavar='IKEPOLICY',
            required=True,
            help=_('IKE policy associated with this connection (name or ID)'))
        parser.add_argument(
            '--ipsecpolicy',
            metavar='IPSECPOLICY',
            required=True,
            help=_('IPsec policy associated with this connection '
                   '(name or ID)'))
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('Set friendly name for the connection'))
        osc_utils.add_project_owner_option_to_parser(parser)
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.network
        attrs = _get_common_attrs(self.app.client_manager, parsed_args)
        if parsed_args.vpnservice:
            _vpnservice_id = client.find_vpn_service(
                parsed_args.vpnservice, ignore_missing=False)['id']
            attrs['vpnservice_id'] = _vpnservice_id
        if parsed_args.ikepolicy:
            _ikepolicy_id = client.find_vpn_ike_policy(
                parsed_args.ikepolicy, ignore_missing=False)['id']
            attrs['ikepolicy_id'] = _ikepolicy_id
        if parsed_args.ipsecpolicy:
            _ipsecpolicy_id = client.find_vpn_ipsec_policy(
                parsed_args.ipsecpolicy, ignore_missing=False)['id']
            attrs['ipsecpolicy_id'] = _ipsecpolicy_id
        if parsed_args.peer_id:
            attrs['peer_id'] = parsed_args.peer_id
        if parsed_args.peer_address:
            attrs['peer_address'] = parsed_args.peer_address
        if parsed_args.psk:
            attrs['psk'] = parsed_args.psk
        if parsed_args.name:
            attrs['name'] = parsed_args.name
        if (bool(parsed_args.local_endpoint_group) !=
                bool(parsed_args.peer_endpoint_group)):
            message = _("You must specify both local and peer endpoint "
                        "groups")
            raise exceptions.CommandError(message)
        if not parsed_args.peer_cidrs and not parsed_args.local_endpoint_group:
            message = _("You must specify endpoint groups or peer CIDR(s)")
            raise exceptions.CommandError(message)
        obj = client.create_vpn_ipsec_site_connection(**attrs)
        display_columns, columns = utils.get_osc_show_columns_for_sdk_resource(
            obj, _attr_map_dict, ['location', 'tenant_id', 'action',
                                  'timeout', 'interval'])
        data = utils.get_dict_properties(obj, columns, formatters=_formatters)
        return display_columns, data


class DeleteIPsecSiteConnection(command.Command):
    _description = _("Delete IPsec site connection(s)")

    def get_parser(self, prog_name):
        parser = super(DeleteIPsecSiteConnection, self).get_parser(prog_name)
        parser.add_argument(
            'ipsec_site_connection',
            metavar='<ipsec-site-connection>',
            nargs='+',
            help=_('IPsec site connection to delete (name or ID)'))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.network
        result = 0
        for ipsec_conn in parsed_args.ipsec_site_connection:
            try:
                ipsec_con_id = client.find_vpn_ipsec_site_connection(
                    ipsec_conn, ignore_missing=False)['id']
                client.delete_vpn_ipsec_site_connection(ipsec_con_id)
            except Exception as e:
                result += 1
                LOG.error(_("Failed to delete IPsec site connection with "
                            "name or ID '%(ipsec_site_conn)s': %(e)s"),
                          {'ipsec_site_conn': ipsec_conn, 'e': e})

        if result > 0:
            total = len(parsed_args.ipsec_site_connection)
            msg = (_("%(result)s of %(total)s IPsec site connection failed "
                     "to delete.") % {'result': result, 'total': total})
            raise exceptions.CommandError(msg)


class ListIPsecSiteConnection(command.Lister):
    _description = _("List IPsec site connections "
                     "that belong to a given project")

    def get_parser(self, prog_name):
        parser = super(ListIPsecSiteConnection, self).get_parser(prog_name)
        parser.add_argument(
            '--long',
            action='store_true',
            default=False,
            help=_("List additional fields in output")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.network
        obj = client.vpn_ipsec_site_connections()
        headers, columns = column_util.get_column_definitions(
            _attr_map, long_listing=parsed_args.long)
        return (headers, (utils.get_dict_properties(
            s, columns, formatters=_formatters) for s in obj))


class SetIPsecSiteConnection(command.Command):
    _description = _("Set IPsec site connection properties")

    def get_parser(self, prog_name):
        parser = super(SetIPsecSiteConnection, self).get_parser(prog_name)
        _get_common_parser(parser)
        parser.add_argument(
            '--peer-id',
            help=_('Peer router identity for authentication. Can be '
                   'IPv4/IPv6 address, e-mail address, key id, or FQDN'))
        parser.add_argument(
            '--peer-address',
            help=_('Peer gateway public IPv4/IPv6 address or FQDN'))
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Set friendly name for the connection'))
        parser.add_argument(
            'ipsec_site_connection',
            metavar='<ipsec-site-connection>',
            help=_('IPsec site connection to set (name or ID)'))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.network
        attrs = _get_common_attrs(self.app.client_manager,
                                  parsed_args, is_create=False)
        if parsed_args.peer_id:
            attrs['peer_id'] = parsed_args.peer_id
        if parsed_args.peer_address:
            attrs['peer_address'] = parsed_args.peer_address
        if parsed_args.name:
            attrs['name'] = parsed_args.name
        ipsec_conn_id = client.find_vpn_ipsec_site_connection(
            parsed_args.ipsec_site_connection, ignore_missing=False)['id']
        try:
            client.update_vpn_ipsec_site_connection(ipsec_conn_id, **attrs)
        except Exception as e:
            msg = (_("Failed to set IPsec site "
                     "connection '%(ipsec_conn)s': %(e)s")
                   % {'ipsec_conn': parsed_args.ipsec_site_connection, 'e': e})
            raise exceptions.CommandError(msg)


class ShowIPsecSiteConnection(command.ShowOne):
    _description = _("Show information of a given IPsec site connection")

    def get_parser(self, prog_name):
        parser = super(ShowIPsecSiteConnection, self).get_parser(prog_name)
        parser.add_argument(
            'ipsec_site_connection',
            metavar='<ipsec-site-connection>',
            help=_('IPsec site connection to display (name or ID)'))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.network
        ipsec_site_id = client.find_vpn_ipsec_site_connection(
            parsed_args.ipsec_site_connection, ignore_missing=False)['id']
        obj = client.get_vpn_ipsec_site_connection(ipsec_site_id)
        display_columns, columns = utils.get_osc_show_columns_for_sdk_resource(
            obj, _attr_map_dict, ['location', 'tenant_id', 'action',
                                  'timeout', 'interval'])
        data = utils.get_dict_properties(obj, columns, formatters=_formatters)
        return (display_columns, data)



# Copyright 2015 Huawei Technologies India Pvt Ltd.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutronclient._i18n import _
from neutronclient.common import utils
from neutronclient.neutron import v2_0 as neutronV20

# key=object_type: value={key=resource, value=cmd_resource}
RBAC_OBJECTS = {'network': {'network': 'network'},
                'qos-policy': {'policy': 'qos_policy'}}


def _get_cmd_resource(obj_type):
    resource = list(RBAC_OBJECTS[obj_type])[0]
    cmd_resource = RBAC_OBJECTS[obj_type][resource]
    return resource, cmd_resource


def get_rbac_obj_params(client, obj_type, obj_id_or_name):
    resource, cmd_resource = _get_cmd_resource(obj_type)
    obj_id = neutronV20.find_resourceid_by_name_or_id(
        client=client, resource=resource, name_or_id=obj_id_or_name,
        cmd_resource=cmd_resource)

    return obj_id, cmd_resource


class ListRBACPolicy(neutronV20.ListCommand):
    """List RBAC policies that belong to a given tenant."""

    resource = 'rbac_policy'
    list_columns = ['id', 'object_type', 'object_id']
    pagination_support = True
    sorting_support = True
    allow_names = False


class ShowRBACPolicy(neutronV20.ShowCommand):
    """Show information of a given RBAC policy."""

    resource = 'rbac_policy'
    allow_names = False


class CreateRBACPolicy(neutronV20.CreateCommand):
    """Create a RBAC policy for a given tenant."""

    resource = 'rbac_policy'

    def add_known_arguments(self, parser):
        parser.add_argument(
            'name',
            metavar='RBAC_OBJECT',
            help=_('ID or name of the RBAC object.'))
        parser.add_argument(
            '--type', choices=RBAC_OBJECTS.keys(),
            required=True,
            type=utils.convert_to_lowercase,
            help=_('Type of the object that RBAC policy affects.'))
        parser.add_argument(
            '--target-tenant',
            default='*',
            help=_('ID of the tenant to which the RBAC '
                   'policy will be enforced.'))
        parser.add_argument(
            '--action', choices=['access_as_external', 'access_as_shared'],
            type=utils.convert_to_lowercase,
            required=True,
            help=_('Action for the RBAC policy.'))

    def args2body(self, parsed_args):
        neutron_client = self.get_client()
        _object_id, _object_type = get_rbac_obj_params(neutron_client,
                                                       parsed_args.type,
                                                       parsed_args.name)
        body = {
            'object_id': _object_id,
            'object_type': _object_type,
            'target_tenant': parsed_args.target_tenant,
            'action': parsed_args.action,
        }
        return {self.resource: body}


class UpdateRBACPolicy(neutronV20.UpdateCommand):
    """Update RBAC policy for given tenant."""

    resource = 'rbac_policy'
    allow_names = False

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--target-tenant',
            help=_('ID of the tenant to which the RBAC '
                   'policy will be enforced.'))

    def args2body(self, parsed_args):
        body = {'target_tenant': parsed_args.target_tenant}
        return {self.resource: body}


class DeleteRBACPolicy(neutronV20.DeleteCommand):
    """Delete a RBAC policy."""

    resource = 'rbac_policy'
    allow_names = False



# Copyright 2017 FUJITSU LIMITED
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import collections
from unittest import mock
import uuid

from openstack.network.v2 import vpn_endpoint_group as vpn_epg
from openstack.network.v2 import vpn_ike_policy as vpn_ikep
from openstack.network.v2 import vpn_ipsec_policy as vpn_ipsecp
from openstack.network.v2 import vpn_ipsec_site_connection as vpn_sitec
from openstack.network.v2 import vpn_service


class FakeVPNaaS(object):

    def create(self, attrs={}):
        """Create a fake vpnaas resources

        :param Dictionary attrs:
            A dictionary with all attributes
        :return:
            A OrderedDict faking the vpnaas resource
        """
        self.ordered.update(attrs)
        if 'IKEPolicy' == self.__class__.__name__:
            return vpn_ikep.VpnIkePolicy(**self.ordered)
        if 'IPSecPolicy' == self.__class__.__name__:
            return vpn_ipsecp.VpnIpsecPolicy(**self.ordered)
        if 'VPNService' == self.__class__.__name__:
            return vpn_service.VpnService(**self.ordered)
        if 'EndpointGroup' == self.__class__.__name__:
            return vpn_epg.VpnEndpointGroup(**self.ordered)

    def bulk_create(self, attrs=None, count=2):
        """Create multiple fake vpnaas resources

        :param Dictionary attrs:
            A dictionary with all attributes
        :param int count:
            The number of vpnaas resources to fake
        :return:
            A list of dictionaries faking the vpnaas resources
        """
        return [self.create(attrs=attrs) for i in range(0, count)]

    def get(self, attrs=None, count=2):
        """Get multiple fake vpnaas resources

        :param Dictionary attrs:
            A dictionary with all attributes
        :param int count:
            The number of vpnaas resources to fake
        :return:
            A list of dictionaries faking the vpnaas resource
        """
        if attrs is None:
            self.attrs = self.bulk_create(count=count)
        return mock.Mock(side_effect=attrs)


class IKEPolicy(FakeVPNaaS):
    """Fake one or more IKE policies"""

    def __init__(self):
        super(IKEPolicy, self).__init__()
        self.ordered = collections.OrderedDict((
            ('id', 'ikepolicy-id-' + uuid.uuid4().hex),
            ('name', 'my-ikepolicy-' + uuid.uuid4().hex),
            ('auth_algorithm', 'sha1'),
            ('encryption_algorithm', 'aes-128'),
            ('ike_version', 'v1'),
            ('pfs', 'group5'),
            ('description', 'my-desc-' + uuid.uuid4().hex),
            ('phase1_negotiation_mode', 'main'),
            ('project_id', 'project-id-' + uuid.uuid4().hex),
            ('lifetime', {'units': 'seconds', 'value': 3600}),
        ))


class IPSecPolicy(FakeVPNaaS):
    """Fake one or more IPsec policies"""

    def __init__(self):
        super(IPSecPolicy, self).__init__()
        self.ordered = collections.OrderedDict((
            ('id', 'ikepolicy-id-' + uuid.uuid4().hex),
            ('name', 'my-ikepolicy-' + uuid.uuid4().hex),
            ('auth_algorithm', 'sha1'),
            ('encapsulation_mode', 'tunnel'),
            ('transform_protocol', 'esp'),
            ('encryption_algorithm', 'aes-128'),
            ('pfs', 'group5'),
            ('description', 'my-desc-' + uuid.uuid4().hex),
            ('project_id', 'project-id-' + uuid.uuid4().hex),
            ('lifetime', {'units': 'seconds', 'value': 3600}),
        ))


class VPNService(FakeVPNaaS):
    """Fake one or more VPN services"""

    def __init__(self):
        super(VPNService, self).__init__()
        self.ordered = collections.OrderedDict((
            ('id', 'vpnservice-id-' + uuid.uuid4().hex),
            ('name', 'my-vpnservice-' + uuid.uuid4().hex),
            ('router_id', 'router-id-' + uuid.uuid4().hex),
            ('subnet_id', 'subnet-id-' + uuid.uuid4().hex),
            ('flavor_id', 'flavor-id-' + uuid.uuid4().hex),
            ('admin_state_up', True),
            ('status', 'ACTIVE'),
            ('description', 'my-desc-' + uuid.uuid4().hex),
            ('project_id', 'project-id-' + uuid.uuid4().hex),
            ('external_v4_ip', '192.0.2.42'),
            ('external_v6_ip', '2001:0db8:207a:4a3a:053b:6fab:7df9:1afd'),
        ))


class EndpointGroup(FakeVPNaaS):
    """Fake one or more Endpoint Groups"""

    def __init__(self):
        super(EndpointGroup, self).__init__()
        self.ordered = collections.OrderedDict((
            ('id', 'ep-group-id-' + uuid.uuid4().hex),
            ('name', 'my-ep-group-' + uuid.uuid4().hex),
            ('type', 'cidr'),
            ('endpoints', ['10.0.0.0/24', '20.0.0.0/24']),
            ('description', 'my-desc-' + uuid.uuid4().hex),
            ('project_id', 'project-id-' + uuid.uuid4().hex),
        ))


class IPsecSiteConnection(object):
    """Fake one or more IPsec site connections"""
    @staticmethod
    def create_conn(attrs=None):
        """Create a fake IPsec conn.

        :param Dictionary attrs:
            A dictionary with all attributes
        :return:
            A Dictionary with id, name, peer_address, auth_mode, status,
            project_id, peer_cidrs, vpnservice_id, ipsecpolicy_id,
            ikepolicy_id, mtu, initiator, admin_state_up, description,
            psk, route_mode, local_id, peer_id, local_ep_group_id,
            peer_ep_group_id
        """
        attrs = attrs or {}

        # Set default attributes.
        conn_attrs = {
            'id': 'ipsec-site-conn-id-' + uuid.uuid4().hex,
            'name': 'my-ipsec-site-conn-' + uuid.uuid4().hex,
            'peer_address': '192.168.2.10',
            'auth_mode': '',
            'status': '',
            'project_id': 'project-id-' + uuid.uuid4().hex,
            'peer_cidrs': [],
            'vpnservice_id': 'vpnservice-id-' + uuid.uuid4().hex,
            'ipsecpolicy_id': 'ipsecpolicy-id-' + uuid.uuid4().hex,
            'ikepolicy_id': 'ikepolicy-id-' + uuid.uuid4().hex,
            'mtu': 1500,
            'initiator': 'bi-directional',
            'admin_state_up': True,
            'description': 'my-vpn-connection',
            'psk': 'abcd',
            'route_mode': '',
            'local_id': '',
            'peer_id': '192.168.2.10',
            'local_ep_group_id': 'local-ep-group-id-' + uuid.uuid4().hex,
            'peer_ep_group_id': 'peer-ep-group-id-' + uuid.uuid4().hex,
        }

        # Overwrite default attributes.
        conn_attrs.update(attrs)
        return vpn_sitec.VpnIPSecSiteConnection(**conn_attrs)



#    (c) Copyright 2013 Hewlett-Packard Development Company, L.P.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import argparse

from neutronclient._i18n import _
from neutronclient.common import utils
from neutronclient.neutron import v2_0 as neutronv20
from neutronclient.neutron.v2_0.vpn import utils as vpn_utils


def add_common_args(parser, is_create=True):
    parser.add_argument(
        '--description',
        help=_('Description of the IKE policy.'))
    parser.add_argument(
        '--auth-algorithm',
        type=utils.convert_to_lowercase,
        default='sha1' if is_create else argparse.SUPPRESS,
        choices=['sha1', 'sha256', 'sha384', 'sha512'],
        help=_('Authentication algorithm, default:sha1.'))
    parser.add_argument(
        '--encryption-algorithm',
        default='aes-128' if is_create else argparse.SUPPRESS,
        type=utils.convert_to_lowercase,
        help=_('Encryption algorithm, default:aes-128.'))
    parser.add_argument(
        '--phase1-negotiation-mode',
        default='main' if is_create else argparse.SUPPRESS,
        choices=['main'],
        type=utils.convert_to_lowercase,
        help=_('IKE Phase1 negotiation mode, default:main.'))
    parser.add_argument(
        '--ike-version',
        default='v1' if is_create else argparse.SUPPRESS,
        choices=['v1', 'v2'],
        type=utils.convert_to_lowercase,
        help=_('IKE version for the policy, default:v1.'))
    parser.add_argument(
        '--pfs',
        default='group5' if is_create else argparse.SUPPRESS,
        type=utils.convert_to_lowercase,
        help=_('Perfect Forward Secrecy, default:group5.'))
    parser.add_argument(
        '--lifetime',
        metavar="units=UNITS,value=VALUE",
        type=utils.str2dict_type(optional_keys=['units', 'value']),
        help=vpn_utils.lifetime_help("IKE"))


def parse_common_args2body(parsed_args, body):
    neutronv20.update_dict(parsed_args, body,
                           ['auth_algorithm', 'encryption_algorithm',
                            'phase1_negotiation_mode', 'ike_version',
                            'pfs', 'name', 'description', 'tenant_id'])
    if parsed_args.lifetime:
        vpn_utils.validate_lifetime_dict(parsed_args.lifetime)
        body['lifetime'] = parsed_args.lifetime
    return body


class ListIKEPolicy(neutronv20.ListCommand):
    """List IKE policies that belong to a tenant."""

    resource = 'ikepolicy'
    list_columns = ['id', 'name', 'auth_algorithm',
                    'encryption_algorithm', 'ike_version', 'pfs']
    _formatters = {}
    pagination_support = True
    sorting_support = True


class ShowIKEPolicy(neutronv20.ShowCommand):
    """Show information of a given IKE policy."""

    resource = 'ikepolicy'
    help_resource = 'IKE policy'


class CreateIKEPolicy(neutronv20.CreateCommand):
    """Create an IKE policy."""

    resource = 'ikepolicy'

    def add_known_arguments(self, parser):
        parser.add_argument(
            'name', metavar='NAME',
            help=_('Name of the IKE policy.'))
        add_common_args(parser)

    def args2body(self, parsed_args):
        return {'ikepolicy': parse_common_args2body(parsed_args, body={})}


class UpdateIKEPolicy(neutronv20.UpdateCommand):
    """Update a given IKE policy."""

    resource = 'ikepolicy'
    help_resource = 'IKE policy'

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--name',
            help=_('Updated name of the IKE policy.'))
        add_common_args(parser, is_create=False)

    def args2body(self, parsed_args):
        return {'ikepolicy': parse_common_args2body(parsed_args, body={})}


class DeleteIKEPolicy(neutronv20.DeleteCommand):
    """Delete a given IKE policy."""

    resource = 'ikepolicy'
    help_resource = 'IKE policy'



# Copyright (c) 2016 Intel Corporation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from neutronclient._i18n import _
from neutronclient.common import utils
from neutronclient.neutron import v2_0 as neutronv20
from neutronclient.neutron.v2_0.qos import rule as qos_rule


MINIMUM_BANDWIDTH_RULE_RESOURCE = 'minimum_bandwidth_rule'


def add_minimum_bandwidth_arguments(parser):
    parser.add_argument(
        '--min-kbps',
        required=True,
        type=str,
        help=_('QoS minimum bandwidth assurance, expressed in kilobits '
               'per second.'))
    # NOTE(ralonsoh): the only direction implemented is "egress". Please,
    # refer to the spec (https://review.opendev.org/#/c/316082/).
    parser.add_argument(
        '--direction',
        # NOTE(ihrachys): though server picks the default for us (egress), it's
        # better to require the argument to make the UI more explicit and the
        # intentions more clear in the future when we add other values for the
        # attribute on server side.
        required=True,
        type=utils.convert_to_lowercase,
        choices=['egress'],
        help=_('Traffic direction.'))


def update_minimum_bandwidth_args2body(parsed_args, body):
    neutronv20.update_dict(parsed_args, body, ['min_kbps', 'direction'])


class CreateQoSMinimumBandwidthRule(qos_rule.QosRuleMixin,
                                    neutronv20.CreateCommand):
    """Create a qos minimum bandwidth rule."""

    resource = MINIMUM_BANDWIDTH_RULE_RESOURCE

    def add_known_arguments(self, parser):
        super(CreateQoSMinimumBandwidthRule, self).add_known_arguments(
            parser)
        add_minimum_bandwidth_arguments(parser)

    def args2body(self, parsed_args):
        body = {}
        update_minimum_bandwidth_args2body(parsed_args, body)
        return {self.resource: body}


class ListQoSMinimumBandwidthRules(qos_rule.QosRuleMixin,
                                   neutronv20.ListCommand):
    """List all qos minimum bandwidth rules belonging to the specified policy.

    """

    resource = MINIMUM_BANDWIDTH_RULE_RESOURCE
    _formatters = {}
    pagination_support = True
    sorting_support = True


class ShowQoSMinimumBandwidthRule(qos_rule.QosRuleMixin,
                                  neutronv20.ShowCommand):
    """Show information about the given qos minimum bandwidth rule."""

    resource = MINIMUM_BANDWIDTH_RULE_RESOURCE
    allow_names = False


class UpdateQoSMinimumBandwidthRule(qos_rule.QosRuleMixin,
                                    neutronv20.UpdateCommand):
    """Update the given qos minimum bandwidth rule."""

    resource = MINIMUM_BANDWIDTH_RULE_RESOURCE
    allow_names = False

    def add_known_arguments(self, parser):
        super(UpdateQoSMinimumBandwidthRule, self).add_known_arguments(
            parser)
        add_minimum_bandwidth_arguments(parser)

    def args2body(self, parsed_args):
        body = {}
        update_minimum_bandwidth_args2body(parsed_args, body)
        return {self.resource: body}


class DeleteQoSMinimumBandwidthRule(qos_rule.QosRuleMixin,
                                    neutronv20.DeleteCommand):
    """Delete a given qos minimum bandwidth rule."""

    resource = MINIMUM_BANDWIDTH_RULE_RESOURCE
    allow_names = False



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from neutronclient._i18n import _
from neutronclient.common import exceptions
from neutronclient.neutron import v2_0 as neutronv20


# List of resources can be set tag
TAG_RESOURCES = ['network', 'subnet', 'port', 'router', 'subnetpool']


def _convert_resource_args(client, parsed_args):
    resource_type = client.get_resource_plural(parsed_args.resource_type)
    resource_id = neutronv20.find_resourceid_by_name_or_id(
        client, parsed_args.resource_type, parsed_args.resource)
    return resource_type, resource_id


def _add_common_arguments(parser):
    parser.add_argument('--resource-type',
                        choices=TAG_RESOURCES,
                        dest='resource_type',
                        required=True,
                        help=_('Resource Type.'))
    parser.add_argument('--resource',
                        required=True,
                        help=_('Resource name or ID.'))


class AddTag(neutronv20.NeutronCommand):
    """Add a tag into the resource."""

    def get_parser(self, prog_name):
        parser = super(AddTag, self).get_parser(prog_name)
        _add_common_arguments(parser)
        parser.add_argument('--tag',
                            required=True,
                            help=_('Tag to be added.'))
        return parser

    def take_action(self, parsed_args):
        client = self.get_client()
        if not parsed_args.tag:
            raise exceptions.CommandError(
                _('Cannot add an empty value as tag'))
        resource_type, resource_id = _convert_resource_args(client,
                                                            parsed_args)
        client.add_tag(resource_type, resource_id, parsed_args.tag)


class ReplaceTag(neutronv20.NeutronCommand):
    """Replace all tags on the resource."""

    def get_parser(self, prog_name):
        parser = super(ReplaceTag, self).get_parser(prog_name)
        _add_common_arguments(parser)
        parser.add_argument('--tag',
                            metavar='TAG',
                            action='append',
                            dest='tags',
                            required=True,
                            help=_('Tag (This option can be repeated).'))
        return parser

    def take_action(self, parsed_args):
        client = self.get_client()
        resource_type, resource_id = _convert_resource_args(client,
                                                            parsed_args)
        body = {'tags': parsed_args.tags}
        client.replace_tag(resource_type, resource_id, body)


class RemoveTag(neutronv20.NeutronCommand):
    """Remove a tag on the resource."""

    def get_parser(self, prog_name):
        parser = super(RemoveTag, self).get_parser(prog_name)
        _add_common_arguments(parser)
        tag_opt = parser.add_mutually_exclusive_group()
        tag_opt.add_argument('--all',
                             action='store_true',
                             help=_('Remove all tags on the resource.'))
        tag_opt.add_argument('--tag',
                             help=_('Tag to be removed.'))
        return parser

    def take_action(self, parsed_args):
        if not parsed_args.all and not parsed_args.tag:
            raise exceptions.CommandError(
                _("--all or --tag must be specified"))
        client = self.get_client()
        resource_type, resource_id = _convert_resource_args(client,
                                                            parsed_args)
        if parsed_args.all:
            client.remove_tag_all(resource_type, resource_id)
        else:
            client.remove_tag(resource_type, resource_id, parsed_args.tag)



#    (c) Copyright 2013 Hewlett-Packard Development Company, L.P.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import argparse

from oslo_serialization import jsonutils

from neutronclient._i18n import _
from neutronclient.common import exceptions
from neutronclient.common import utils
from neutronclient.neutron import v2_0 as neutronv20
from neutronclient.neutron.v2_0.vpn import utils as vpn_utils


def _format_peer_cidrs(ipsec_site_connection):
    try:
        return '\n'.join([jsonutils.dumps(cidrs) for cidrs in
                          ipsec_site_connection['peer_cidrs']])
    except (TypeError, KeyError):
        return ''


class ListIPsecSiteConnection(neutronv20.ListCommand):
    """List IPsec site connections that belong to a given tenant."""

    resource = 'ipsec_site_connection'
    _formatters = {'peer_cidrs': _format_peer_cidrs}
    list_columns = [
        'id', 'name', 'peer_address', 'auth_mode', 'status']
    pagination_support = True
    sorting_support = True


class ShowIPsecSiteConnection(neutronv20.ShowCommand):
    """Show information of a given IPsec site connection."""

    resource = 'ipsec_site_connection'
    help_resource = 'IPsec site connection'


class IPsecSiteConnectionMixin(object):

    def add_known_arguments(self, parser, is_create=True):
        parser.add_argument(
            '--name',
            help=_('Set friendly name for the connection.'))
        parser.add_argument(
            '--description',
            help=_('Set a description for the connection.'))
        parser.add_argument(
            '--dpd',
            metavar="action=ACTION,interval=INTERVAL,timeout=TIMEOUT",
            type=utils.str2dict_type(
                optional_keys=['action', 'interval', 'timeout']),
            help=vpn_utils.dpd_help("IPsec connection."))
        parser.add_argument(
            '--local-ep-group',
            help=_('Local endpoint group ID/name with subnet(s) for '
                   'IPSec connection.'))
        parser.add_argument(
            '--peer-ep-group',
            help=_('Peer endpoint group ID/name with CIDR(s) for '
                   'IPSec connection.'))
        parser.add_argument(
            '--peer-cidr',
            action='append', dest='peer_cidrs',
            help=_('[DEPRECATED in Mitaka] Remote subnet(s) in CIDR format. '
                   'Cannot be specified when using endpoint groups. Only '
                   'applicable, if subnet provided for VPN service.'))
        parser.add_argument(
            '--peer-id',
            required=is_create,
            help=_('Peer router identity for authentication. Can be '
                   'IPv4/IPv6 address, e-mail address, key id, or FQDN.'))
        parser.add_argument(
            '--peer-address',
            required=is_create,
            help=_('Peer gateway public IPv4/IPv6 address or FQDN.'))
        parser.add_argument(
            '--psk',
            required=is_create,
            help=_('Pre-shared key string.'))
        parser.add_argument(
            '--mtu',
            default='1500' if is_create else argparse.SUPPRESS,
            help=_('MTU size for the connection, default:1500.'))
        parser.add_argument(
            '--initiator',
            default='bi-directional' if is_create else argparse.SUPPRESS,
            choices=['bi-directional', 'response-only'],
            help=_('Initiator state in lowercase, default:bi-directional'))

    def args2body(self, parsed_args, body=None):
        """Add in conditional args and then return all conn info."""

        if body is None:
            body = {}
        if parsed_args.dpd:
            vpn_utils.validate_dpd_dict(parsed_args.dpd)
            body['dpd'] = parsed_args.dpd
        if parsed_args.local_ep_group:
            _local_epg = neutronv20.find_resourceid_by_name_or_id(
                self.get_client(), 'endpoint_group',
                parsed_args.local_ep_group)
            body['local_ep_group_id'] = _local_epg
        if parsed_args.peer_ep_group:
            _peer_epg = neutronv20.find_resourceid_by_name_or_id(
                self.get_client(), 'endpoint_group',
                parsed_args.peer_ep_group)
            body['peer_ep_group_id'] = _peer_epg
        if hasattr(parsed_args, 'mtu') and int(parsed_args.mtu) < 68:
            message = _("Invalid MTU value: MTU must be "
                        "greater than or equal to 68.")
            raise exceptions.CommandError(message)
        # ToDo (Reedip) : Remove below check when peer-cidr is removed
        if parsed_args.peer_cidrs and parsed_args.local_ep_group:
            message = _("You cannot specify both endpoint groups and peer "
                        "CIDR(s).")
            raise exceptions.CommandError(message)
        neutronv20.update_dict(parsed_args, body,
                               ['peer_id', 'mtu', 'initiator', 'psk',
                                'peer_address', 'name', 'description',
                                'peer_cidrs'])
        return {self.resource: body}


class CreateIPsecSiteConnection(IPsecSiteConnectionMixin,
                                neutronv20.CreateCommand):
    """Create an IPsec site connection."""
    resource = 'ipsec_site_connection'

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--admin-state-down',
            default=True, action='store_false',
            help=_('Set admin state up to false.'))
        parser.add_argument(
            '--vpnservice-id', metavar='VPNSERVICE',
            required=True,
            help=_('VPN service instance ID associated with this connection.'))
        parser.add_argument(
            '--ikepolicy-id', metavar='IKEPOLICY',
            required=True,
            help=_('IKE policy ID associated with this connection.'))
        parser.add_argument(
            '--ipsecpolicy-id', metavar='IPSECPOLICY',
            required=True,
            help=_('IPsec policy ID associated with this connection.'))
        super(CreateIPsecSiteConnection, self).add_known_arguments(parser)

    def args2body(self, parsed_args):
        _vpnservice_id = neutronv20.find_resourceid_by_name_or_id(
            self.get_client(), 'vpnservice',
            parsed_args.vpnservice_id)
        _ikepolicy_id = neutronv20.find_resourceid_by_name_or_id(
            self.get_client(), 'ikepolicy',
            parsed_args.ikepolicy_id)
        _ipsecpolicy_id = neutronv20.find_resourceid_by_name_or_id(
            self.get_client(), 'ipsecpolicy',
            parsed_args.ipsecpolicy_id)
        body = {
            'vpnservice_id': _vpnservice_id,
            'ikepolicy_id': _ikepolicy_id,
            'ipsecpolicy_id': _ipsecpolicy_id,
            'admin_state_up': parsed_args.admin_state_down,
        }
        if parsed_args.tenant_id:
            body['tenant_id'] = parsed_args.tenant_id

        if (bool(parsed_args.local_ep_group) !=
                bool(parsed_args.peer_ep_group)):
            message = _("You must specify both local and peer endpoint "
                        "groups.")
            raise exceptions.CommandError(message)
        if not parsed_args.peer_cidrs and not parsed_args.local_ep_group:
            message = _("You must specify endpoint groups or peer CIDR(s).")
            raise exceptions.CommandError(message)
        return super(CreateIPsecSiteConnection, self).args2body(parsed_args,
                                                                body)


class UpdateIPsecSiteConnection(IPsecSiteConnectionMixin,
                                neutronv20.UpdateCommand):
    """Update a given IPsec site connection."""

    resource = 'ipsec_site_connection'
    help_resource = 'IPsec site connection'

    def add_known_arguments(self, parser):
        utils.add_boolean_argument(
            parser, '--admin-state-up',
            help=_('Update the administrative state. (True meaning "Up")'))
        super(UpdateIPsecSiteConnection, self).add_known_arguments(parser,
                                                                   False)

    def args2body(self, parsed_args):
        body = {}
        neutronv20.update_dict(parsed_args, body, ['admin_state_up'])
        return super(UpdateIPsecSiteConnection, self).args2body(parsed_args,
                                                                body)


class DeleteIPsecSiteConnection(neutronv20.DeleteCommand):
    """Delete a given IPsec site connection."""

    resource = 'ipsec_site_connection'
    help_resource = 'IPsec site connection'
